/** \file
 * Hello world example app.
 * A very simple mpi skeleton application.
 */

#include "helloapp.h"
#include <stdio.h>

namespace hellocomm {

  // Public constructor
  // Creates a reference counted app object
  boost::shared_ptr<helloapp>
  helloapp::construct() {
    boost::shared_ptr<helloapp> obj(new helloapp);
    obj->set_self(obj);
    return obj;
  }

  // Get a copy 
  // Launchers can use this to create app threads 
  boost::shared_ptr<sstmac::mpiapp>
  helloapp::clone() {
    boost::shared_ptr<helloapp> obj(new helloapp);
    obj->set_self(obj);
    return obj;
  }

  // Overrides thread::run virtual function
  // The task thread context is started by a call into this function
  void helloapp::run() { 

    // initialize mpi
    if(mpi() == 0)
      throw sstmac::nullerror("helloapp:  mpiapi pointer is null.");
    sstmac::mpicomm world(mpi()->comm_world());
    sstmac::mpiid rank(world.rank());
    sstmac::timestamp start = mpi()->init();
    std::cout << "multmpi[" << rank << "]::run(): mpi()->init() at " 
              << start << "\n";

    // simulate broadcasting 1 double from rank 0 with no real payload
    const int count=1;
    const sstmac::mpiid root(0);
    mpi()->bcast(count, sstmac::mpitype::mpi_double, root, world);
    if(rank == sstmac::mpiid(0))
      std::cout << rank << ": Hello COMM_WORLD.\n";
    else
      std::cout << rank << ": Hello mpiid(id=0).\n";

    // finalize 
    sstmac::timestamp done = mpi()->finalize();
    std::cout << "multmpi[" << rank << "]::run():  mpi()->finalize() at " 
              << done << "\n";
  }

} // end of namespace hellosim
