## @file
#  Python script which runs pyexample::app.
#  A very simple script to run the pyexample::app example
#  skeleton mpi application within the SST/Macro simulator.

import sstmac
import pyexample

print "=================== running pyexample ==================="

#--------------------------------------
# Section 1. Set up simulation manager
#--------------------------------------

mgr = sstmac.manager.construct()

#--------------------------------------
# Section 2. Set up interconnect model
#--------------------------------------

# Construct network topology (fat tree).
network = sstmac.fattree.construct(4,18,1,1)

# Construct an interconnect model (sharedcircuit)
# and give it a bandwidth allocation model (flower).
latency = 1e-6    # latency in seconds
bandwidth = 10e9  # bandwidth in bytes/second
flow = sstmac.ordered_flower.construct(network,bandwidth,bandwidth)
interconnect = sstmac.sharedcircuit.construct(sstmac.timestamp(latency),
                                              bandwidth,network,flow)
mgr.set_interconnect(interconnect)

#----------------------
# Section 3. Add nodes
#----------------------

# Add a node to the manager, manager will clone it "nodes" times
# and assign those nodes to network slots.
nodes = 16
procpernode = 1
memory_bandwidth = 1e10  # bandwidth in bytes/second
mgr.add_nodes(sstmac.simplenode.construct(memory_bandwidth), nodes)
print ("nodes = %d procpernode = %d") % (nodes,procpernode)

#-------------------------------------
# Section 4. Add application launcher
#-------------------------------------

# Construct an application object.
myapp = pyexample.app.construct()

# Construct a launcher.
launcher = sstmac.mpilaunch.construct(myapp)

starttime = sstmac.timestamp(0.)
mgr.add_launcher(starttime, launcher, nodes, procpernode)

#-------------------------------
# Section 5. Run the simulation
#-------------------------------

time = mgr.run()
runtime = time.sec();
print ("simulated run time for %d nodes with %d processes per node = %12.9f seconds") % (nodes,procpernode,runtime)
