/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sst/core/element.h>
#include <sstmac/macrobridge/macrorecver.h>
#include <sstmac/macrobridge/macrosender.h>
#include <iostream>

template <class COMPONENT>
static SST::Component* create_component(SST::ComponentId_t id, SST::Component::Params_t &params)
{
  std::cerr << "allocating component" << std::endl;
  return new COMPONENT(id, params);
}


static const SST::ElementInfoComponent components[] = {
  { "recver",
    "Receives events for SST/macro from SST/core",
    0,
    create_component<sstmac::macrorecver>,
  },
  { "sender",
    "Sends events from SST/macro to SST/core.",
    0,
    create_component<sstmac::macrosender>,
  },
  {0, 0, 0, 0}
};

extern "C" {
  SST::ElementLibraryInfo sstmacbridge_eli = {
      "sstmacbridge",
      "SST/macro components",
      components,
  };
}
