/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/macrobridge/macrobridge.h>
#include <sstmac/simulator/timestamp.h>
#include <sst/core/eventFunctor.h>
#include <sst/core/link.h>
#include <sst/core/event.h>
#include <sst/core/timeLord.h>
//#include <Python.h>
#include <iostream>
#include <exception>

using namespace SST;

struct sucks : public std::exception {
  const std::string what_;
  sucks(const std::string &life) : what_(life) {
    std::cerr << "Throwing error: " << what_ << "\n";
  }
  virtual ~sucks() throw() {}
  const char *what() const throw() { return what_.c_str(); }
};

struct macroevent_setup : public Event {
  virtual ~macroevent_setup() {}
};

//
// Private method to handle component events.
//
void macrobridge::handle_events(Event *evt) {
  std::cerr << "macrobridge::handle_events(" << evt << ")\n";
  delete evt;
}

//
// Private method to add wiring between components.
//
void macrobridge::get_wired(const std::string &target) {
  // Templates gone wild.
  typedef Event::Handler<macrobridge> handler_t;
  handler_t *handler = new handler_t(this, &macrobridge::handle_events);
  port_ = configureLink(target, defaultTimeBase, handler);
  if(port_ == NULL) {
    throw sucks("macrobridge::get_wired:  LinkAdd failed.");
  }
}

macrobridge::macrobridge(ComponentId_t id, Params_t &params) :
  Component(id)
{
  registerTimeBase(sstmac::timestamp::tick_interval_string());

  std::cerr << "macrobridge[" << this << "] constructing.\n"
            << "Parameters[" << params.size() << "]\n";
  std::string target = "";
  for(Params_t::const_iterator it = params.begin(); it != params.end(); ++it) {
    if(it->first == "dest") target = it->second;
    std::cerr << "    \"" << it->first << "\"==>\"" << it->second << "\"\n";
  }
  if(target == "") {
    throw sucks("macrobridge::macrobridge:  Target name is undefined.");
  }
  this->get_wired(target);

  // Run a python interpreter for the provided script.
  /*
  if(params.find("script") != params.end()) {
    PyInteractiveFlag = 0;
    PyInitialize();
    PyRun_SimpleString("import sys\n");
    // eventually we will load the SST/macroscale library here.
    // PyRun_SimpleString("sys.path = sys.path + <sstmac pylibdir>\n");
    // PyRun_SimpleString("import sstmac\n");
  }
  */
  std::cerr << "macrobridge[" << this << "] constructed\n";
}

macrobridge::~macrobridge() {
  std::cerr << "macrobridge[" << this << "] going out of scope\n";
}

//
// Git goin'
//
int macrobridge::Setup() {
  std::cerr << "macrobridge[" << this << "]::Setup()\n";
  for(double time = 0; time < 0.004; time += 0.01) {
    macroevent_setup *evt = new macroevent_setup;
    port_->Send(time, evt);
  }
  return 0;
}

//
// Whoa!
//
int macrobridge::Finish() {
  std::cerr << "macrobridge[" << this << "]::Finish()\n";
  return 0;
}

//BOOST_CLASS_EXPORT(macrobridge)

//BOOST_CLASS_EXPORT_TEMPLATE4(SST::EventHandler,
//                             macrobridge, bool, SST::Time_t, SST::Event *)
