/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_MACROBRIDGE_MACROBRIDGE_H
#define SSTMAC_MACROBRIDGE_MACROBRIDGE_H

#include <sstmac/sst/sst_config.h>
#include <sstmac/sst/core/component.h>
#include <sstmac/sst/core/link.h>


/**
 * A bridge component to instantiate a bridge between SST/macroscale
 * and SST/microscale.  The bridge will allow SST/macroscale to run
 * over the SST/microscale event manager and (eventually) transfer
 * events between the two scales.
 *
 * This initial test doesn't really do anything terribly interesting
 * (mainly for discovery and debugging purposes).
 */
class macrobridge : public SST::Component {
  /// The port wiring this component to others.
  SST::Link *port_;

private:
  /// Copy constructor is blocked so we can identify possible copy issues.
  macrobridge(const macrobridge&) : 
    Component(*this)
  {}
  
  /// Assignment operator is blocked to identify possible copy issues.
  void operator=(const macrobridge&) {}

  /// Private method to handle component events.
  void handle_events(SST::Event *evt);

  /// Private method to set up the wiring between components.
  void get_wired(const std::string &target);

public:
  /// This constructor is called by the SST runtime framework.
  macrobridge(SST::ComponentId_t id,
              SST::Component::Params_t &params);

  virtual ~macrobridge();

  /// As far as I can tell, this method gets called by the runtime system
  /// after setup is complete (makes perfect sense).
  int Setup();

  /// This should be what happens when we're all done (although you never know).
  int Finish();
};

#endif // ! SSTMAC_MACROBRIDGE_MACROBRIDGE_H
