/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <boost/shared_ptr.hpp>
#include <sstmac/network/composite.h>
#include <sstmac/network/product.h>
#include <sstmac/network/network.h>
#include <string>
#include <sstream>
#include <iostream>


namespace sstmac {
  composite::composite(const boost::shared_ptr<network>& n1,
                       const boost::shared_ptr<network>& n2) {
    assemble(n1,n2);

    std::ostringstream ostr;
    ostr << "Composite" << "(" << n1->name() << "," << n2->name() << ")";
    name_ = ostr.str();
  }

  boost::shared_ptr<composite>
  composite::construct(const boost::shared_ptr<network>& n1,
            const boost::shared_ptr<network>& n2) {
    return boost::shared_ptr<composite>(new composite(n1, n2));
  }
  
  composite::~composite() {
  }

  std::string
  composite::name() const {
    return name_;
  }
}
