/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// crossbar.h: Interface for crossbar networks.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_NETWORK_CROSSBAR_H
#define SSTMAC_NETWORK_CROSSBAR_H

#include <sstmac/network/network.h>

namespace sstmac {

  /** The crossbar network generates a network which connects
      all nodes with only two hops: those to and from the crossbar.
      The inputs for this network can specify the following attributes:

      <table border="1">

      <tr><td>Keyword<td>Type<td>Default<td>Description

      <tr><td><tt>n</tt><td>int<td>256<td>The radix of the crossbar.

      </table>
  */

  class crossbar: public network {
  private:
    vertex_index sw_index_;
  protected:
    crossbar(int nnode);

  public:
    static boost::shared_ptr<crossbar> construct(int nnode=256);
    virtual ~crossbar();

    void shortest_path(const vertex&v1, const vertex&v2, walk&w) const;
    int64_t diameter() const;
    double a() const;
    int64_t b() const;
    bool vertex_symmetric() const;
    bool is_switch(vertex_index v) const;
    std::string name() const;
  };

}

#endif
