/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// line.h: Interface for line (aka 1D mesh) networks.
//
// Author: (ring) Curtis Janssen <cljanss@ca.sandia.gov>
// Adaption to 1D mesh by David Evensky <evensky@sandia.gov>

#ifndef SSTMAC_NETWORK_LINE_H
#define SSTMAC_NETWORK_LINE_H


#include <sstmac/network/network.h>

namespace sstmac {

  /// This implements a line network.
  class line: public network {
  public:
    bool mesh_routing;
  private:
    int nx_;
    void init_line(int nx);
  protected:
    line(int nx);
    //line(TiXmlHandle);

  public:
    static boost::shared_ptr<line> construct(int nodes);
    virtual ~line();

    void shortest_path(const vertex &v1, const vertex &v2, walk &) const;
    int64_t diameter() const;
    int64_t b() const;
    bool vertex_symmetric() const;
    std::string name() const;
  };

}

#endif
