/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef NETSTAT_H_
#define NETSTAT_H_

#include <sstmac/simulator/stats.h>

#include <boost/shared_ptr.hpp>

namespace sstmac
{
  class netstat : public stats
  {
  public:
    netstat();
    virtual
    ~netstat();

    static boost::shared_ptr<netstat>
       construct()
       {
         boost::shared_ptr<netstat> retval(new netstat());
         return retval;
       }

       void
       trackBytes(double f);

       void trackHops(int64_t hops);


       double getBytes(){ return bytesTransferred;}

       double getAverageHopCount(){return hopCount / numHopCount;}

     protected:
       double bytesTransferred;
       double hopCount;
       double numHopCount;  // for calculated the average hop count;
  };

}

#endif /* NETSTAT_H_ */
