/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/network/netstatmanager.h>

namespace sstmac
{

  netstatmanager::netstatmanager(std::string fname) : statsmanager(fname)
  {
    // TODO Auto-generated constructor stub

  }

  netstatmanager::~netstatmanager()
  {
    // TODO Auto-generated destructor stub
  }

  double
  netstatmanager::getAverageHopCount()
  {

    list<boost::shared_ptr<netstat> >::iterator it;
    double ret = 0;
    int num = 0;
    for (it = stats.begin(); it != stats.end(); it++)
      {
        ret += (*it)->getAverageHopCount();
        num++;
      }
    return ret / num;
  }

  double
  netstatmanager::getTotalBytes()
  {

    list<boost::shared_ptr<netstat> >::iterator it;
    double ret = 0;
    int num = 0;
    for (it = stats.begin(); it != stats.end(); it++)
      {
        ret += (*it)->getBytes();
        num++;
      }
    return ret;
  }

  void
  netstatmanager::printSummary(const timestamp &t)
  {

    ostream &out = getOutputStream();

    out << "------ Network (t=" << t.sec() << ") -------" << std::endl;
    out << "Totals:" << std::endl;
    out << "\t average hops per message: " << getAverageHopCount() << " "
        << std::endl;
    out << "\t bytes sent: " << getTotalBytes() << " " << std::endl;

    out << "------------------" << std::endl;
  }

}
