/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef NETSTATMANAGER_H_
#define NETSTATMANAGER_H_

#include <sstmac/simulator/statsmanager.h>
#include <string>
#include <list>
#include <sstmac/network/netstat.h>

using namespace std;

namespace sstmac
{
  class netstatmanager : public statsmanager
  {
  private:
    netstatmanager(std::string fname);

    list<boost::shared_ptr<netstat> > stats;

  public:
    virtual
    ~netstatmanager();

    static boost::shared_ptr<netstatmanager>
    construct(std::string fname)
    {
      boost::shared_ptr<netstatmanager> ret(new netstatmanager(fname));
      return ret;
    }

    // Let the manager know about this stat, so that he can collect it
    void
    registerStat(boost::shared_ptr<netstat> &stat)
    {
      stats.push_back(stat);
    }

    double
    getAverageHopCount();
    double
    getTotalBytes();

    // prints the power stats
    virtual void
    printSummary(const timestamp &t);

  };

}

#endif /* PROCSTATMANAGER_H_ */
