/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// tbird.h: Class for the tbird network.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_NETWORK_TBIRD_H
#define SSTMAC_NETWORK_TBIRD_H

#include <sstmac/network/network.h>

namespace sstmac {

  /** The tbird network generates an oversubscribed fat tree network like
      that used in the tbird machine. */
  class tbird: public network {
    void init_tbird();

  protected:
    //tbird(TiXmlHandle);
    tbird();

  public:
    static boost::shared_ptr<tbird> construct();
    virtual ~tbird();

    void shortest_path(const vertex&v1, const vertex&v2, walk&w) const;
    void route(const vertex &v1, const vertex &v2, walk &w) const;
    int64_t diameter() const;
    double a() const;
    int64_t b() const;
    bool vertex_symmetric() const;
    std::string name() const;
  };

}

#endif
