/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/applaunch-completion.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hello.
  //
  applaunch::completion::completion(const shared_ptr<applaunch>& parent) :
    parent_(parent)
  {}

  //
  // Build an object.
  //
  shared_ptr<applaunch::completion>
  applaunch::completion::construct(const shared_ptr<applaunch>& parent) {
    return shared_ptr<completion>(new completion(parent));
  }

  //
  // Goodbye.
  //
  applaunch::completion::~completion() throw() {
  }

  //
  // Signal an event.
  //
  void applaunch::completion::handle(const timestamp &current_time) {
    parent_->scheduler_exit(current_time);
  }

} // end of namespace sstmac.
