/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MESSAGEID_H
#define SSTMAC_PROCESS_MESSAGEID_H

#include <iostream>

namespace sstmac {

  /**
   * A ligthweight wrapper for a message index.
   * Done to avoid annoying mistakes where index values are mixed up.
   * 
   * I can't wait for C++ to add opaque typedefs.
   */
  struct messageid {
    // These get pretty large, and we need to avoid collisions
    // (otherwise, pending ack requests can start clobbering each other)
    int64_t id;
    explicit messageid(int64_t v = -1) : id(v) {}

    inline messageid& operator++() {
      ++id;
      return *this;
    }
    inline messageid operator++(int) {
      messageid other(*this);
      ++id;
      return other;
    }
  };

  inline bool operator==(const messageid &a, const messageid &b) {
    return (a.id == b.id);
  }
  inline bool operator!=(const messageid &a, const messageid &b) {
    return (a.id != b.id);
  }
  inline bool operator<(const messageid &a, const messageid &b) {
    return (a.id < b.id);
  }
  inline bool operator<=(const messageid &a, const messageid &b) {
    return (a.id <= b.id);
  }
  inline bool operator>(const messageid &a, const messageid &b) {
    return (a.id > b.id);
  }
  inline bool operator>=(const messageid &a, const messageid &b) {
    return (a.id >= b.id);
  }
  inline std::ostream& operator<<(std::ostream &os, const messageid &n) {
    return (os << "messageid(" << n.id << ")");
  }

} // end of namespace sstmac


#endif // ! SSTMAC_PROCESS_MESSAGEID_H
