/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorealltoallv.h>
#include <sstmac/process/mpidirectalltoall.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicorealltoallv::mpicorealltoallv() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicorealltoallv> mpicorealltoallv::construct() {
    return shared_ptr<mpicorealltoallv>(new mpicorealltoallv);
  }

  //
  // Must be virtual.
  //
  mpicorealltoallv::~mpicorealltoallv() throw() {
  }

  //
  // Build a kernel.
  //
  shared_ptr<mpikernel>
  mpicorealltoallv::build(const shared_ptr<processrequest>& thekey,
                          const shared_ptr<mpiqueue>& queue,
                          const std::vector<int> &sendcnts,
                          const mpitype &sendtype,
                          const std::vector<int> &recvcnts,
                          const mpitype &recvtype,
                          const mpitag &tag, const mpicomm &comm,
                          const shared_ptr<const payload>& content,
                          const shared_ptr<mpimagic>& magic) const
  {
    return mpidirectalltoall::construct
      (thekey, queue, sendcnts, sendtype, recvcnts, recvtype,
       tag, comm, content, magic);
  }

} // end of namespace sstmac
