/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOREBARRIER_H
#define SSTMAC_PROCESS_MPICOREBARRIER_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  /**
   * Synchronize nodes.
   *
   * Performed using approximately the mechanism from mpich2-1.0.8
   * (see util/hfmiterator.h).
   */
  class mpicorebarrier : public mpibarrierstrategy {
  protected:
    /// Hello.
    mpicorebarrier();

  public:
    /// Hi.
    static boost::shared_ptr<mpicorebarrier> construct();

    /// Goodbye.
    virtual ~mpicorebarrier() throw();
    
    /// Build a kernel to synchronize nodes.
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<processrequest>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          const mpitag &tag, const mpicomm &comm,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOREBARRIER_H
