/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorebcast.h>
#include <sstmac/process/mpibcastengine.h>
#include <sstmac/util/btscatteriterator.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hello.
  //
  mpicorebcast::mpicorebcast() {
  }

  //
  // Hi.
  //
  shared_ptr<mpicorebcast> mpicorebcast::construct() {
    return shared_ptr<mpicorebcast>(new mpicorebcast);
  }

  //
  // Goodbye.
  //
  mpicorebcast::~mpicorebcast() throw() {
  }

  //
  // Build a kernel to perform a broadcast.
  //
  shared_ptr<mpikernel>
  mpicorebcast::build(const boost::shared_ptr<processrequest>& thekey,
                      const boost::shared_ptr<mpiqueue>& queue,
                      int count, const mpitype &type, const mpiid &root,
                      const mpitag &tag, const mpicomm &comm,
                      const boost::shared_ptr<const payload>& content,
                      const boost::shared_ptr<mpimagic>& /*magic ignored*/) const
  {
    // Here we might want to pick different iterators depending on message size.
    shared_ptr<topologyiterator> iter =
      btscatteriterator::construct(comm.rank().id, comm.size().id, root.id);
    return mpibcastengine::construct
      (iter, thekey, queue, count, type, root, tag, comm, content);
  }
  
} // end of namespace sstmac
