/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicoregatherv.h>
#include <sstmac/process/mpidirectgatherv.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  //
  // Hi.
  //
  mpicoregatherv::mpicoregatherv() {
  }

  //
  // Hello.
  //
  boost::shared_ptr<mpicoregatherv> mpicoregatherv::construct() {
    return boost::shared_ptr<mpicoregatherv>(new mpicoregatherv);
  }

  //
  // Perform a fused reduce-gatherv operation
  //
  boost::shared_ptr<mpikernel>
  mpicoregatherv::build(const boost::shared_ptr<processrequest>& thekey,
                        const boost::shared_ptr<mpiqueue>& queue,
                        int sendcount, const mpitype &sendtype,
                        const std::vector<int> &recvcount,
                        const mpitype &recvtype,
                        const mpiid &root, const mpitag &tag,
                        const mpicomm &comm,
                        const boost::shared_ptr<const payload>& content,
                        const boost::shared_ptr<mpimagic>& magic) const
  {
    return mpidirectgatherv::construct(thekey, queue, sendcount, sendtype,
                                       recvcount, recvtype, root, tag, comm,
                                       content, magic);
  }

} // end of namespace sstmac
