/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOREGATHERV_H
#define SSTMAC_PROCESS_MPICOREGATHERV_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  //
  // Default strategy for gatherv operations.
  //
  class mpicoregatherv : public mpigathervstrategy {
    /// Hi.
    mpicoregatherv();
  public:
    /// Hello.
    static boost::shared_ptr<mpicoregatherv> construct();
    /// Must be virtual.
    virtual ~mpicoregatherv() throw() {}
    /// Perform a fused reduce-gatherv operation
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<processrequest>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          int sendcounts, const mpitype &sendtype,
          const std::vector<int> &recvcount, const mpitype &recvtype,
          const mpiid &root, const mpitag &tag,
          const mpicomm &comm, const boost::shared_ptr<const payload>& content,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOREGATHERV_H
