/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICORERECV_H
#define SSTMAC_PROCESS_MPICORERECV_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  /// Basic receive strategy.  Upon receipt, the receiver will test
  /// whether the sender wanted a return receipt etc. and respond accordingly.
  class mpicorerecv : public mpirecvstrategy {
  protected:
    /// Hello.
    mpicorerecv();
  public:
    /// Hello.
    static boost::shared_ptr<mpicorerecv> construct();
    /// Goodbye.
    virtual ~mpicorerecv() throw();
    /// Post a receive request from the given node to be matched by the
    /// given request handle.  All receives are non-blocking; it is up
    /// to the strategy to handle return receipts or other synchronization.
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<processrequest>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          int count, const mpitype &type, const mpiid &source,
          const mpitag &tag, const mpicomm &comm,
          const boost::shared_ptr<mpimagic>& magic) const;
  };



} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPICORERECV_H
