/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorereduce.h>
#include <sstmac/process/mpirabenseifner.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicorereduce::mpicorereduce() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicorereduce> mpicorereduce::construct() {
    return shared_ptr<mpicorereduce>(new mpicorereduce);
  }

  //
  // Perform a fused reduce-reduce operation
  //
  shared_ptr<mpikernel>
  mpicorereduce::build(const shared_ptr<processrequest>& thekey, const shared_ptr<mpiqueue>& queue,
                       int count, const mpitype &type, const mpiop &op,
                       const mpiid &root, const mpitag &tag,
                       const mpicomm &comm, const shared_ptr<const payload>& content,
                       const shared_ptr<mpimagic>& magic) const
  {
    return mpirabenseifner::construct(false, thekey, queue, count, type, op,
                                      root, tag, comm, content, magic);
  }

} // end of namespace sstmac
