/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorereduce_scatter.h>
#include <sstmac/process/mpihalvingreducescatter.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicorereduce_scatter::mpicorereduce_scatter() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicorereduce_scatter> mpicorereduce_scatter::construct() {
    return shared_ptr<mpicorereduce_scatter>(new mpicorereduce_scatter);
  }

  //
  // Goodbye
  //
  mpicorereduce_scatter::~mpicorereduce_scatter() throw() {
  }

  //
  // Perform a fused reduce-scatter operation
  //
  shared_ptr<mpikernel>
  mpicorereduce_scatter::build(const shared_ptr<processrequest>& thekey,
                               const shared_ptr<mpiqueue>& queue,
                               const std::vector<int> &recvcnts,
                               const mpitype &type, const mpiop &op,
                               const mpitag &tag, const mpicomm &comm,
                               const shared_ptr<const payload>& content,
                               const shared_ptr<mpimagic>& magic) const
  {
    return mpihalvingreducescatter::construct(thekey, queue, recvcnts, type, op,
                                              tag, comm, content, magic);
  }

} // end of namespace sstmac
