/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOREREDUCE_SCATTER_H
#define SSTMAC_PROCESS_MPICOREREDUCE_SCATTER_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  //
  /// Base strategy for fused reduce-scatter operations.
  //
  class mpicorereduce_scatter : public mpireduce_scatterstrategy {
  protected:
    /// Hi.
    mpicorereduce_scatter();
  public:
    /// Hello.
    static boost::shared_ptr<mpicorereduce_scatter> construct();

    /// Goodbye
    virtual ~mpicorereduce_scatter() throw();

    /// Build a kernel to perform a fused reduce-scatter operation
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<processrequest>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          const std::vector<int> &recvcnts, const mpitype &type,
          const mpiop &op, const mpitag &tag, const mpicomm &comm,
          const boost::shared_ptr<const payload>& content,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

}

#endif // ! SSTMAC_PROCESS_MPICOREREDUCE_SCATTER_H
