/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorescan.h>
#include <sstmac/process/mpiscanengine.h>
#include <sstmac/util/rditerator.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  // 
  // Hello.
  //
  mpicorescan::mpicorescan() {
  }

  //
  // Build a new object.
  //
  shared_ptr<mpicorescan> mpicorescan::construct() {
    return shared_ptr<mpicorescan>(new mpicorescan);
  }

  //
  // Destructor.
  //
  mpicorescan::~mpicorescan() throw() {
  }

  //
  // Build a kernel to perform a scan operation
  //
  shared_ptr<mpikernel>
  mpicorescan::build(const boost::shared_ptr<processrequest>& thekey,
                     const boost::shared_ptr<mpiqueue>& queue,
                     int count, const mpitype &type, const mpiop &op,
                     const mpitag &tag, const mpicomm &comm,
                     const shared_ptr<const payload>& content,
                     const boost::shared_ptr<mpimagic>& /*magic ignored*/) const
  {
    shared_ptr<topologyiterator> iter =
      rditerator::construct(comm.rank().id, comm.size().id);
    return mpiscanengine::construct(iter, thekey, queue, count, type,
                                    op, tag, comm, content);
  }

} // end of namespace sstmac

