/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICORESCATTER_H
#define SSTMAC_PROCESS_MPICORESCATTER_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  //
  // Default strategy for scatter operations.
  //
  class mpicorescatter : public mpiscatterstrategy {
    /// Hi.
    mpicorescatter();
  public:
    /// Hello.
    static boost::shared_ptr<mpicorescatter> construct();
    /// Must be virtual.
    virtual ~mpicorescatter() throw() {}
    /// Perform a fused reduce-scatter operation
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<processrequest>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          int sendcount, const mpitype &sendtype,
          int recvcount, const mpitype &recvtype,
          const mpiid &root, const mpitag &tag,
          const mpicomm &comm,
          const boost::shared_ptr<const mpicollpayload>& cc,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICORESCATTER_H
