/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorescatterv.h>
#include <sstmac/process/mpidirectscatterv.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  //
  // Hi.
  //
  mpicorescatterv::mpicorescatterv() {
  }

  //
  // Hello.
  //
  boost::shared_ptr<mpicorescatterv> mpicorescatterv::construct() {
    return boost::shared_ptr<mpicorescatterv>(new mpicorescatterv);
  }

  //
  // Perform a fused reduce-scatterv operation
  //
  boost::shared_ptr<mpikernel>
  mpicorescatterv::build(const boost::shared_ptr<processrequest>& thekey,
                         const boost::shared_ptr<mpiqueue>& queue,
                         const std::vector<int> &sendcount,
                         const mpitype &sendtype,
                         int recvcount, const mpitype &recvtype,
                         const mpiid &root, const mpitag &tag,
                         const mpicomm &comm,
                         const boost::shared_ptr<const mpicollpayload>& content,
                         const boost::shared_ptr<mpimagic>& /*magic ignored*/) const
  {
    return mpidirectscatterv::construct(thekey, queue, sendcount, sendtype,
                                        recvcount, recvtype, root, tag, comm,
                                        content);
  }

} // end of namespace sstmac
