/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicoressend.h>
#include <sstmac/process/mpisendengine.h>

namespace sstmac {

  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hello.
  //
  mpicoressend::mpicoressend() {
  }

  //
  // Build.
  //
  boost::shared_ptr<mpicoressend> mpicoressend::construct() {
    return shared_ptr<mpicoressend>(new mpicoressend);
  }

  //
  // Goodbye.
  //
  mpicoressend::~mpicoressend() throw() {
  }

  //
  // Basic nonblocking send.
  //
  boost::shared_ptr<mpikernel>
  mpicoressend::build(const boost::shared_ptr<processrequest>& thekey,
                      const boost::shared_ptr<mpiqueue>& queue,
                      int count, const mpitype &type, const mpiid &dest,
                      const mpitag &tag, const mpicomm &comm,
                      const boost::shared_ptr<const payload>& content,
                      const boost::shared_ptr<mpimagic>& /*magic is ignored*/) const
  {
    bool rsend = false;
    bool ssend = true;
    return mpisendengine::construct(thekey, queue, count, type, dest, tag, comm,
                                    rsend, ssend, mpimessage::user, content);
  }

} // end of namespace sstmac.
