/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpikernel.h>
#include <sstmac/process/processrequest.h>
#include <sstmac/process/mpistatus.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Constructor 2.
  //
  mpikernel::mpikernel(const boost::shared_ptr<processrequest> &req) :
    when_done_(req)
  {
    if(debugger::active())
      debug() << "mpikernel[" << this << "]::mpikernel():  "
              << "This kernel is associated with processrequest "
              << when_done_ << "\n";
    if(! when_done_) {
      std::cerr << "ERROR:  mpikernel(shared_ptr<processrequest>):  "
                << "NULL process request pointer\n";
      throw valueerror("mpikernel(shared_ptr<processrequest>):  "
                       "NULL process request pointer.");
    }
  }

  //
  // Get a request handle refering to this kernel.
  //
  shared_ptr<processrequest> mpikernel::get_request() {
    return when_done_;
  }

  //
  // This is what happens when the request is complete.
  //
  void mpikernel::complete(const timestamp &current_time,
                           const boost::shared_ptr<mpistatus>& stat)
  {
    if(debugger::active())
      debug() << "mpikernel[" << this << "]::complete(" << current_time
              << ", " << stat << ")\n";
    when_done_->complete(current_time, stat);
    kernel::complete(current_time);
  }

} // end of namespace sstmac
