/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpiop.h>

namespace sstmac {

  static int nextid = 0;

  ///// Reduction operations.
  mpiop::mpiop(const std::string &labelit, const timestamp &costit) :
    label(labelit), cost_per_entry(costit), id(nextid++)
  {}

  //
  // Common reduction operations.
  // For simplicity, I am assuming that reductions take 1 ns/op
  const mpiop mpiop::max("max", timestamp::exact_nsec(1));

  const mpiop mpiop::min("min", timestamp::exact_nsec(1));

  const mpiop mpiop::sum("sum", timestamp::exact_nsec(1));

  const mpiop mpiop::prod("sum", timestamp::exact_nsec(1));

  const mpiop mpiop::land("land", timestamp::exact_nsec(1));

  const mpiop mpiop::band("band", timestamp::exact_nsec(1));

  const mpiop mpiop::lor("lor", timestamp::exact_nsec(1));

  const mpiop mpiop::bor("bor", timestamp::exact_nsec(1));

  const mpiop mpiop::lxor("lxor", timestamp::exact_nsec(1));

  const mpiop mpiop::bxor("bxor", timestamp::exact_nsec(1));
  
  const mpiop
  mpiop::minloc("minloc", timestamp::exact_nsec(1));

  const mpiop
  mpiop::maxloc("maxloc", timestamp::exact_nsec(1));

  const mpiop
  mpiop::replace("replace",timestamp::exact_nsec(1));
  
  const mpiop
  mpiop::op_null("op_null",timestamp::exact_nsec(1));

} // end of namespace sstmac
