/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIPROBE_H
#define SSTMAC_PROCESS_MPIPROBE_H

#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  // Important types.
  class mpistatus;
  class scheduler;
  class key;

  /**
   * A base for event handlers that manages probe requests.
   */
  class mpiprobe : public eventhandler {
    /// Are we done.
    bool done_;
    /// The envelope for this probe
    boost::shared_ptr<mpistatus> status_;
    /// Our scheduler.
    boost::shared_ptr<scheduler> sched_;
    /// The key to unblock the caller.
    boost::shared_ptr<key> key_;

  protected:
    /// Hello.
    mpiprobe(const boost::shared_ptr<scheduler>& sched,
             const boost::shared_ptr<key>& thekey);

  public:
    /// Hi.
    static boost::shared_ptr<mpiprobe>
    construct(const boost::shared_ptr<scheduler>& sched,
              const boost::shared_ptr<key>& thekey);

    /// Goodbye.
    virtual ~mpiprobe() throw();

    /// Test whether we're done.
    bool is_done() const { return done_; }

    /// Set the status associated with this probe request.
    void set_mpistatus(const boost::shared_ptr<mpistatus>& status);

    /// Get the status associated with this probe request.
    boost::shared_ptr<mpistatus> get_mpistatus();

    /// All done.
    void handle(const timestamp &now);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIPROBE_H
