/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIQUEUE_RECVREQUEST_H
#define SSTMAC_PROCESS_MPIQUEUE_RECVREQUEST_H

#include <sstmac/process/mpiqueue.h>
#include <sstmac/util/nocopy.h>

namespace sstmac {

  /**
   * A nested type to handle individual mpi receive requests.
   */
  class mpiqueue::recvrequest : virtual public nocopy {
    /// This object.
    boost::weak_ptr<recvrequest> self_;

    /// The queue to whom we belong.
    boost::weak_ptr<mpiqueue> queue_;

    /// State flag to see whether we are in an ack/send cycle.
    bool acked_;

    /// The parameters I will be matching on.
    mpiid source_;
    mpitag tag_;
    mpicomm comm_;
    mpimessage::category cat_;

    /// The parameters I will not be matching on, but are good error checks.
    int count_;
    mpitype type_;

    /// The object that will be notified upon completion.
    boost::shared_ptr<mpirecv> completion_;

  protected:
    /// Hello.
    recvrequest(const boost::shared_ptr<mpiqueue>& queue, int count,
                const mpitype &type, const mpiid &source, const mpitag &tag,
                const mpicomm &comm, mpimessage::category cat,
                const boost::shared_ptr<mpirecv>& completion);

    /// Who am I?
    void set_self(const boost::shared_ptr<recvrequest>& self);

    /// This is who I am.
    boost::shared_ptr<recvrequest> get_self();

  public:
    /// Hello.
    static boost::shared_ptr<recvrequest>
    construct(const boost::shared_ptr<mpiqueue>& queue, int count,
              const mpitype &type, const mpiid &source, const mpitag &tag,
              const mpicomm &comm, mpimessage::category cat,
              const boost::shared_ptr<mpirecv>& completion);

    /// Goodbye.
    virtual ~recvrequest();

    /// Do we match the given message?
    inline bool matches(const boost::shared_ptr<mpimessage>& msg) const {
      return ((cat_ == msg->cat()) && (comm_.id() == msg->commid()) &&
              source_.matches(msg->source()) && tag_.matches(msg->tag()));
    }

    /// We be done.
    void handle(const timestamp &current_time,
                const boost::shared_ptr<mpimessage>& mess);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPIQUEUE_RECVREQUEST_H
