/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPISENDENGINE_H
#define SSTMAC_PROCESS_MPISENDENGINE_H

#include <sstmac/process/mpikernel.h>
#include <sstmac/process/mpisend.h>
#include <sstmac/process/mpimessage.h>

namespace sstmac {

  // Important types.
  class mpiqueue;
  class payload;

  /*
   * The engine (kernel) that performs a non-synchronous MPI send.
   */
  class mpisendengine : virtual public mpikernel, virtual public mpisend {
    /// This object.
    boost::weak_ptr<mpisendengine> self_;
    boost::shared_ptr<mpiqueue> queue_;
    int count_;
    mpitype type_;
    mpiid dest_;
    mpitag tag_;
    mpicomm comm_;
    bool rsend_;
    bool ssend_;
    mpimessage::category cat_;
    boost::shared_ptr<const payload> content_;

  protected:
    /// Constructor -- sets up our payload to send.
    mpisendengine(const boost::shared_ptr<processrequest>& thekey,
                  const boost::shared_ptr<mpiqueue>& queue,
                  int count, const mpitype &type, const mpiid &dest, 
                  const mpitag &tag, const mpicomm &comm,
                  bool rsend, bool ssend, mpimessage::category cat,
                  const boost::shared_ptr<const payload>& content);

    /// Who am I?
    void set_self(const boost::shared_ptr<mpisendengine>& self);

    /// This is who I am.
    boost::shared_ptr<mpisendengine> get_self();

  public:
    /// Build me a sharable object.
    static boost::shared_ptr<mpisendengine>
    construct(const boost::shared_ptr<processrequest>& thekey,
              const boost::shared_ptr<mpiqueue>& queue,
              int count, const mpitype &type, const mpiid &dest, 
              const mpitag &tag, const mpicomm &comm,
              bool rsend, bool ssend, mpimessage::category cat,
              const boost::shared_ptr<const payload>& content);

    /// Goodbye.
    virtual ~mpisendengine() throw();

    /// This is the method from the kernel.
    virtual void start(const timestamp &start_time,
                       const boost::shared_ptr<nodeinterface>& my_node);

    /// Grab the completion message.
    virtual void handle(const timestamp &current_time);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPISENDENGINE_H
