/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPISERVER_LOCALTRANSFER_H
#define SSTMAC_PROCESS_MPISERVER_LOCALTRANSFER_H

#include <sstmac/process/mpiserver.h>
#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  /**
   * A nested event handler that signals a local message after calling
   * nodeinterface::copy().
   */
  class mpiserver::localtransfer : public eventhandler {
    /// The queue we will be notifying.
    boost::shared_ptr<mpiqueue> queue_;
    /// The payload we will be delivering to the queue.
    boost::shared_ptr<mpimessage> payload_;
    /// We need to let the sender know that their job is done.
    boost::shared_ptr<eventhandler> senddone_;
    
  protected:
    /// Hi.
    localtransfer(const boost::shared_ptr<mpiqueue>& queue,
                  const boost::shared_ptr<mpimessage>& payload,
                  const boost::shared_ptr<eventhandler>& senddone);

  public:
    /// Hello.
    static boost::shared_ptr<localtransfer>
    construct(const boost::shared_ptr<mpiqueue>& queue,
              const boost::shared_ptr<mpimessage>& payload,
              const boost::shared_ptr<eventhandler>& senddone);

    /// Goodbye.
    virtual ~localtransfer() throw();

    /// Event.
    void handle(const timestamp &completion_time);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPISERVER_LOCALTRANSFER_H
