/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpitype.h>
#include <iostream>

namespace sstmac {

  static int nextid = 0;

  //
  // MPI datatypes.
  //
  mpitype::mpitype(const std::string &labelit, const int sizeit) :
    label(labelit), size(sizeit), id(nextid++)
  {}

  //
  // Test for (in)equality.
  //
  bool operator==(const mpitype &a, const mpitype &b) {
    return ((a.size == b.size) && (a.label == b.label));
  }
  bool operator!=(const mpitype &a, const mpitype &b) {
    return !(a == b);
  }

  //
  // Fairly self-explanatory.
  //
  std::ostream& operator<<(std::ostream &os, const mpitype &type) {
    os << "mpitype(label=\"" << type.label << "\", size=" << type.size << ")";
    return os;
  }

  //
  // The data types defined by the MPI standard.
  //
  const mpitype mpitype::mpi_null("mpi_null", 0);
  const mpitype mpitype::mpi_char("mpi_char", 1);
  const mpitype mpitype::mpi_signed_char("mpi_signed_char", 1);
  const mpitype mpitype::mpi_wchar("mpi_wchar", 2);
  const mpitype mpitype::mpi_unsigned_long_long("mpi_unsigned_long_long", 8);
  const mpitype mpitype::mpi_lb("mpi_lb", 0);
  const mpitype mpitype::mpi_ub("mpi_ub", 0);
  const mpitype mpitype::mpi_unsigned_char("mpi_unsigned_char", 1);
  const mpitype mpitype::mpi_byte("mpi_byte", 1);
  const mpitype mpitype::mpi_short("mpi_short", 2);
  const mpitype mpitype::mpi_unsigned_short("mpi_unsigned_short", 2);
  const mpitype mpitype::mpi_int("mpi_int", 4);
  const mpitype mpitype::mpi_unsigned("mpi_unsigned", 4);
  const mpitype mpitype::mpi_long("mpi_long", 8);
  const mpitype mpitype::mpi_unsigned_long("mpi_unsigned_long", 8);
  const mpitype mpitype::mpi_float("mpi_float", 4);
  const mpitype mpitype::mpi_double("mpi_double", 8);
  const mpitype mpitype::mpi_long_double("mpi_long_double", 16);
  const mpitype mpitype::mpi_long_long_int("mpi_long_long_int", 8);
  const mpitype mpitype::mpi_long_long("mpi_long_long", 8);
  const mpitype mpitype::mpi_packed("mpi_packed", 1);
  const mpitype mpitype::mpi_float_int("mpi_float_int", 8);
  const mpitype mpitype::mpi_double_int("mpi_double_int", 12);
  const mpitype mpitype::mpi_long_int("mpi_long_int", 8);
  const mpitype mpitype::mpi_short_int("mpi_short_int", 2);
  const mpitype mpitype::mpi_2int("mpi_2int", 8);
  const mpitype mpitype::mpi_long_double_int("mpi_long_double_int", 20);
  const mpitype mpitype::mpi_complex("mpi_complex", 8);
  const mpitype mpitype::mpi_double_complex("mpi_double_complex", 16);
  const mpitype mpitype::mpi_logical("mpi_logical", 1);
  const mpitype mpitype::mpi_real("mpi_real", 4);
  const mpitype mpitype::mpi_double_precision("mpi_double_precision", 8);
  const mpitype mpitype::mpi_integer("mpi_integer", 4);
  const mpitype mpitype::mpi_2integer("mpi_2integer", 8);
  const mpitype mpitype::mpi_2complex("mpi_2complex", 16);
  const mpitype mpitype::mpi_2double_complex("mpi_2double_complex", 32);
  const mpitype mpitype::mpi_2real("mpi_2real", 8);
  const mpitype mpitype::mpi_2double_precision("mpi_2double_precision", 16);
  const mpitype mpitype::mpi_character("mpi_character", 1);

} // end of namespace sstmac
