/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MULTICOMPLETE_HANDLER_H
#define SSTMAC_PROCESS_MULTICOMPLETE_HANDLER_H

#include <sstmac/process/multicomplete.h>
#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  /**
   * An event handler that allows a multicomplete object to wait on
   * multiple requests at once.
   */
  class multicomplete::handler : public eventhandler {
    /// This object.
    boost::weak_ptr<handler> self_;
    /// The object that bore us.
    boost::shared_ptr<multicomplete> parent_;
    /// The index of this request in the parent.
    int64_t index_;

  protected:
    /// Hello.
    handler(const boost::shared_ptr<multicomplete>& parent, int64_t index);

    /// Who am I?
    void set_self(const boost::shared_ptr<handler>& self);

    /// This is who I am.
    boost::shared_ptr<handler> get_self();

  public:
    /// New and improved.
    static boost::shared_ptr<handler>
    construct(const boost::shared_ptr<multicomplete>& parent, int64_t index);

    /// Goodbye.
    virtual ~handler() throw();

    /// All done.
    virtual void handle(const timestamp &current_time);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MULTICOMPLETE_HANDLER_H
