/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_RECVHANDLER_H
#define SSTMAC_PROCESS_RECVHANDLER_H

#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/simulator/ppid.h>
#include <sstmac/simulator/networkinterface.h>

namespace sstmac {

  // Important types.
  class networkdata;
  class node;

  /**
   * An event handler that signals an incoming receive operation
   * by calling node::incoming_message and passing it the event.
   */
  class recvhandler : public eventhandler {
  private:
    boost::shared_ptr<networkinterface> nic_;
    ppid   ppid_;
    nodeid sender_;
    boost::shared_ptr<networkdata> event_;

  protected:
    /// The handler has to know to know what node to signal
    /// and what event to send.
    recvhandler(const boost::shared_ptr<networkinterface>& the_nic,
                ppid   the_ppid,
                nodeid the_sender,
                const boost::shared_ptr<networkdata>& the_event) :
      nic_(the_nic), ppid_(the_ppid), sender_(the_sender), event_(the_event)
    {}

  public:
    /// Build an object and return a shared_ptr to it.
    static boost::shared_ptr<recvhandler>
    construct(const boost::shared_ptr<networkinterface>& the_nic,
              ppid   the_ppid,
              nodeid the_sender,
              const boost::shared_ptr<networkdata>& the_event)
    {
      return boost::shared_ptr<recvhandler>
        (new recvhandler(the_nic, the_ppid, the_sender, the_event));
    }

    /// Goodbye.
    virtual ~recvhandler() throw() {}

    /// Complete the request.
    void handle(const timestamp &completion_time) {
      nic_->incoming_message(completion_time, ppid_, sender_, event_);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_RECVHANDLER_H
