/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_RECVREQUEST_H
#define SSTMAC_PROCESS_RECVREQUEST_H

#include <sstmac/simulator/timestamp.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Forward definition of important types.
  class networkdata;

  /**
   * A request for data to be received.
   */
  class recvrequest {
  public:
    virtual ~recvrequest() throw() {}

    /// Test whether this request matches a given message.
    virtual bool matches(const boost::shared_ptr<const networkdata>& message) = 0;

    /// Complete a receive.
    /// Throw valueerror if this message does not match the request.
    virtual void complete(const timestamp &completion_time,
                          const boost::shared_ptr<const networkdata>& message) = 0;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_RECVREQUEST_H
