/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_RUNINTERPOLATOR_H
#define SSTMAC_PROCESS_RUNINTERPOLATOR_H

#include <sstmac/simulator/timestamp.h>
#include <boost/shared_ptr.hpp>
#include <map>

namespace sstmac {

  /**
   * The beginnings of an interpolator to guesstimate runtimes.
   * Right now, this is very specific for minimd, but it should be
   * made more general.
   */
  class runinterpolator {
    /// Private container to hold timing info (possibly sub-indexed).
    //class timeblock;
    
    /// Base (grid) parameters to use for the interpolation.
    /// I need to look into PCA or other similar analysis methods
    /// for efficient interpolation.
    

    /// Input parameters (set using set_*_parameter).
    std::map<std::string, int> iprm_;
    std::map<std::string, double> rprm_;

    /// Cached (interpolated) value parameters.
    std::map<std::string, int> ival_;
    std::map<std::string, double> rval_;

    /// Interpolated guestimates of runtime for different code blocks.
    //std::map<std::string, timeblock> runtime_;

  public:
    /// Get a copy.
    boost::shared_ptr<runinterpolator> clone() {
      return boost::shared_ptr<runinterpolator>(new runinterpolator);
    }

    /// Define an integer input parameter.
    void set_int_parameter(const std::string &label, int value) {
      iprm_[label] = value;
    }

    /// Define a real-valued input parameter.
    void set_real_parameter(const std::string &label, double value) {
      rprm_[label] = value;
    }

    /// Perform interpolation to the given parameters.
    /// This should (if possible) cache values for future parameter requests.
    /// All required input paramters must be defined before calling initialize.
    void initialize() {
      
    }

    /// Get the aggregate time associated with the labelled function.
    timestamp get(const std::string &label) {
      (void)label;
      return timestamp(0);
    }

    /// Get the indexed time for the labelled function.
    timestamp get(const std::string &label, int index) {
      (void)label; (void)index;
      return timestamp(0);
    }

    /// Get the sub-indexed time for the labelled function.
    timestamp get(const std::string &label, int index, int subindex) {
      (void)label; (void)index; (void)subindex;
      return timestamp(0);
    }

    /// Get the interpolated guess-value for the given variable.
    int get_int(const std::string &label) {
      (void)label;
      return 0;
    }

    /// Get an interpolated value based on a loop index.
    int get_int(const std::string &label, int iteration) {
      (void)label; (void)iteration;
      return 0;
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_RUNINTERPOLATOR_H
