/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/threadwake.h>
#include <sstmac/process/scheduler.h>
#include <sstmac/util/errors.h>
#include <stdlib.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hello.
  //
  threadwake::threadwake(const boost::shared_ptr<scheduler>& s,
                         const boost::shared_ptr<key>& k) :
    sched_(s), key_(k), done_(false)
  {}

  //
  // Hello.
  //
  boost::shared_ptr<threadwake>
  threadwake::construct(const boost::shared_ptr<scheduler>& s,
                        const boost::shared_ptr<key>& k)
  {
    return shared_ptr<threadwake>(new threadwake(s, k));
  }

  //
  // Goodbye.
  //
  threadwake::~threadwake() throw() {
  }

  //
  // Are we done?
  //
  bool threadwake::done() const {
    return done_;
  }

  //
  // Go.
  //
  void threadwake::handle(const timestamp &) {
    sched_.lock()->unblock(key_);
  }

} // end of namespace sstmac
