/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_THREADWAKE_H
#define SSTMAC_PROCESS_THREADWAKE_H

#include <sstmac/simulator/eventhandler.h>
#include <boost/weak_ptr.hpp>

namespace sstmac {

  // Important types.
  class scheduler;
  class key;

  /**
   * An event handler that unblocks the given key upon completion.
   */
  class threadwake : public eventhandler {
    // Try to avoid cyclic reference counts.
    boost::weak_ptr<scheduler> sched_;
    boost::shared_ptr<key> key_;
    bool done_;

  protected:
    /// Hello.
    threadwake(const boost::shared_ptr<scheduler>& s, const boost::shared_ptr<key>& k);

  public:
    /// Hello.
    static boost::shared_ptr<threadwake>
    construct(const boost::shared_ptr<scheduler>& s, const boost::shared_ptr<key>& k);

    /// Goodbye.
    virtual ~threadwake() throw();

    /// Test whether this operation has been completed.
    bool done() const;

    /// Go.
    void handle(const timestamp &completion_time);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_THREADWAKE_H
