/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESSOR_EVENTDATA_H
#define SSTMAC_PROCESSOR_EVENTDATA_H

#include <sstmac/processor/instructions.h>
#include <boost/shared_ptr.hpp>
#include <string>
#include <map>
#include <stdint.h>

namespace sstmac {

  /**
   * Input for processor models that use
   * performance counter data
   */
  class eventdata : public instructions {

    eventdata();
    void set_self( const boost::shared_ptr<eventdata>& );

  public:

    static boost::shared_ptr<eventdata>  
      construct();   

    ~eventdata() {}

    void
    set_event(std::string key, uint64_t val)
    { data[key]=val; }

    uint64_t get_event(std::string key);

    bool has_event(std::string key);

  private:

    boost::shared_ptr<eventdata> self_;
    std::map<std::string,uint64_t> data;

  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESSOR_EVENTDATA_H
