/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESSOR_PROCESSOR_H
#define SSTMAC_PROCESSOR_PROCESSOR_H

#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/eventmanager.h>
#include <sstmac/simulator/ppid.h>
#include <sstmac/processor/instructions.h>
#include <sstmac/simulator/powermanager.h>
#include <boost/shared_ptr.hpp>
#include <sstmac/processor/processor_config.h>
#include <sstmac/simulator/has_stats.h>
#include <sstmac/processor/procstats.h>
#include <sstmac/simulator/powerstat.h>

namespace sstmac
{

  /**
   * An interface for processor models
   */
  class processor : public has_stats
  {
  public:
    processor(boost::shared_ptr<processor_config> cfg)
    {
      proc_cfg = cfg;
    }
    /// Goodbye.
    virtual
    ~processor()
    {
    }

    /// Get a new processor model of the same type as the current one.
    virtual boost::shared_ptr<processor>
    clone() const = 0;

    /// Process a compute kernel.
    /// \param inst instructions object that is processed.
    /// \return timestamp time to process instructions.
    virtual timestamp
    compute(const boost::shared_ptr<instructions>& inst) = 0;

    boost::shared_ptr<processor_config>
    getProcCfg()
    {
      return proc_cfg;
    }

    virtual list<boost::shared_ptr<stats> >
    getStats()
    {
      list<boost::shared_ptr<stats> > ret;
      ret.push_back(pstats);
      ret.push_back(procpower);
      return ret;
    }

  protected:
    boost::shared_ptr<processor_config> proc_cfg;

    // stats
    boost::shared_ptr<powerstat> procpower;
    boost::shared_ptr<procstats> pstats;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESSOR_PROCESSOR_H
