/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef PROCSTATMANAGER_H_
#define PROCSTATMANAGER_H_

#include <sstmac/simulator/statsmanager.h>
#include <string>
#include <sstmac/processor/procstats.h>
#include <list>

using namespace std;

namespace sstmac
{
  class procstatmanager : public statsmanager
  {
  private:
    procstatmanager(std::string fname);

    list<boost::shared_ptr<procstats> > stats;

  public:
    virtual
    ~procstatmanager();

    static boost::shared_ptr<procstatmanager>
    construct(std::string fname)
    {
      boost::shared_ptr<procstatmanager> ret(new procstatmanager(fname));
      return ret;
    }

    // Let the manager know about this stat, so that he can collect it
    void
    registerStat(boost::shared_ptr<procstats> &stat)
    {
      stats.push_back(stat);
    }

    double
    getAverageFPS(const timestamp &t);

    // prints the power stats
    virtual void
    printSummary(const timestamp &t);

  };

}

#endif /* PROCSTATMANAGER_H_ */
