/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/processor/simpleprocessor-config.h>
#include <sstmac/util/errors.h>



namespace sstmac
{

  //
  // Protected constructor.
  //
  simpleprocessor_config::simpleprocessor_config() : processor_config(processor_config::PROC_SIMPLE),
    flops_per_sec(1E6), pwr_static(20), pwr_per_sec(20)
  {
  }

  //
  // Protected constructor using strings.
  //
  simpleprocessor_config::simpleprocessor_config(double flops, double pwr_s,
      double pwr_p) : processor_config(processor_config::PROC_SIMPLE),
    flops_per_sec(flops_per_sec), pwr_static(pwr_s), pwr_per_sec(pwr_p)
  {
  }

  //
  // Public default constructor.
  //
  boost::shared_ptr<simpleprocessor_config>
  simpleprocessor_config::construct()
  {
    boost::shared_ptr<simpleprocessor_config> retval(
        new simpleprocessor_config());
    return retval;
  }

  //
  // Public constructor.
  //
  boost::shared_ptr<simpleprocessor_config>
  simpleprocessor_config::construct(double flops, double pwr_s, double pwr_p)
  {
    boost::shared_ptr<simpleprocessor_config> retval(
        new simpleprocessor_config(flops, pwr_s, pwr_p));
    return retval;
  }



} // end of namespace sstmac
