/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESSOR_SIMPLEPROCESSOR_CONFIG_H
#define SSTMAC_PROCESSOR_SIMPLEPROCESSOR_CONFIG_H

#include <boost/shared_ptr.hpp>
#include <string>
#include <sstmac/simulator/timestamp.h>

#include <sstmac/processor/processor_config.h>

namespace sstmac
{

  /**
   * Store configuration info for simple processor models.
   */
  class simpleprocessor_config : public processor_config
  {
  private:


    // Parameters
    double flops_per_sec; //how many flops get done per second

    double pwr_static; // in watts
    double pwr_per_sec; //how much DYNAMIC ENERGY (in joules) is dissipated
    //per second when we're computing

  protected:
    /// Protected constructor. Initializes to default configuration.
    simpleprocessor_config();

    /// Protected constructor using strings.
    simpleprocessor_config(double flops, double pwr_s, double pwr_p);

  public:
    /// Public default constructor.
    static boost::shared_ptr<simpleprocessor_config>
    construct();

    /// Public constructor. 
    static boost::shared_ptr<simpleprocessor_config>
    construct(double flops, double pwr_s, double pwr_p);


    /// Farewell.
    ~simpleprocessor_config() throw ()
    {
    }

    double
    get_fps()
    {
      return flops_per_sec;
    }
    double
    get_pwr_static()
    {
      return pwr_static;
    }
    double
    get_per_sec()
    {
      return pwr_per_sec;
    }

  };

} // end of namespace sstmac

#endif // SSTMAC_PROCESSOR_SIMPLEPROCESSOR_CONFIG_H
