/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESSOR_SIMPLEPROCESSOR_H
#define SSTMAC_PROCESSOR_SIMPLEPROCESSOR_H

#include <sstmac/processor/processor.h>
#include <sstmac/simulator/ppid.h>
#include <sstmac/simulator/timestamp.h>
#include <sstmac/simulator/eventhandler.h>
#include <boost/shared_ptr.hpp>


#include <sstmac/simulator/power_config.h>
#include <sstmac/simulator/powermanager.h>

namespace sstmac
{

  // Forward declaration of important types.
  class simpleprocessor_config;
  class instructions;

  /**
   * A very simple processor model
   */
  class simpleprocessor : public processor, virtual public nocopy
  {

  private:

    boost::shared_ptr<simpleprocessor_config> config_;

    /// This object.
    boost::weak_ptr<simpleprocessor> self_;





  protected:

    /// Protected constructor.
    simpleprocessor(const boost::shared_ptr<simpleprocessor_config>& cfg);

    ///
    /// Who am I?
    ///
    void
    set_self(const boost::shared_ptr<simpleprocessor>& self);

  public:

    /// Goodbye.
    ~simpleprocessor()
    {
    }

    /// Create simple processor model.
    /// \Param cfg model configuration object
    static boost::shared_ptr<simpleprocessor>
    construct(const boost::shared_ptr<simpleprocessor_config>& cfg);

    /// Inherited from processor.
    boost::shared_ptr<processor>
    clone() const;

    /// Process a compute kernel.
    /// \param inst is the instructions object that will be run.
    /// \return timestamp time to process instructions.
    timestamp
    compute(const boost::shared_ptr<instructions>& inst);


  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESSOR_SIMPLEPROCESSOR_H
