/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/blockindexing.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  using boost::shared_ptr;

  //
  // Default constructor.
  //
  blockindexing::blockindexing() {
  }
  
  blockindexing::~blockindexing() throw() {
  }

  //
  // Static method to allocate object.
  //
  shared_ptr<blockindexing> blockindexing::construct() {
    return shared_ptr<blockindexing>(new blockindexing);
  }

  void blockindexing::allocate(const std::vector<nodeid> &nodes,
                               int ppn,
                               std::vector<nodeid> &result,
                               int nproc)
  {
    if(nodes.empty()) {
      throw valueerror("blockindexing::allocate:  Empty node vector.");
    }
    if(ppn <= 0) {
      throw valueerror("blockindexing::allocate:  ppn must be > 0.");
    }
    if(ppn*nodes.size()<nproc) {
      throw valueerror("blockindexing::allocate:  require ppn*nodes.size()>=nproc");
    }
    result.clear();
    std::vector<nodeid>::const_iterator it, end = nodes.end();
    for(it = nodes.begin(); it != end; ++it) {
      int64_t this_ppn;
      if (result.size() + ppn > nproc) this_ppn = nproc - result.size();
      else this_ppn = ppn;
      result.insert(result.end(), this_ppn, *it);
    }
  }

} // end of namespace sstmac
