/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_BLOCKINDEXING_H
#define SSTMAC_SIMULATOR_BLOCKINDEXING_H

#include <sstmac/simulator/indexstrategy.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * An index strategy that allocates indices using a round robin.
   */
  class blockindexing : public indexstrategy {
  protected:
    /// Protected constructor -- default policy for initialization of objects
    /// is to use the construct methods.
    blockindexing();
  public:
    /// Construct a new object of this type.
    static boost::shared_ptr<blockindexing> construct();

    virtual ~blockindexing() throw();
    
    /// Perform block allocation.
    /// This method does not test the uniqueness of input nodes.
    /// \param nodes is the list of unique nodes to be used for the allocation
    /// \param ppn is the maximum number of processes allocated on any node.
    /// \param result is the resulting vector of nodes (size nproc)
    /// \param nproc the total number of processes to allocate
    /// \throw valueerror if nodes.empty()
    /// \throw valueerror if ppn <= 0
    /// \throw valueerror if nodes.size()*ppn < nproc
    virtual void allocate(const std::vector<nodeid> &nodes,
                          int ppn,
                          std::vector<nodeid> &result,
                          int nproc);

  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_BLOCKINDEXING_H
