/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_ENVIRONMENT_H
#define SSTMAC_SIMULATOR_ENVIRONMENT_H

#include <sstmac/simulator/timestamp.h>
#include <sstmac/simulator/ppid.h>
#include <sstmac/simulator/taskid.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/util/nocopy.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Forward declaration of relevant types.
  class kernel;
  class nodeinterface;
  class tasklist;
  class node;

  /**
   * Base class to give a process access to its parallel environment.
   */
  class environment : public nocopy {
  public:
    virtual ~environment() {}

    /// Get the parallel process id associated with this run.
    virtual ppid get_ppid() const = 0;

    /// Get the mapping of tasks onto nodes.
    virtual boost::shared_ptr<tasklist> get_tasklist() const = 0;

    /// Get the node delegate corresponding to the given task index.
    virtual boost::shared_ptr<nodeinterface> get(const taskid &tid) const = 0;

    /// Get the node corresponding to the given task.
    /// \param tid is the task id
    /// \return the node object for the task
    virtual boost::shared_ptr<node> get_node(const taskid &tid) const = 0;
    
    /// Notify the runtime system that this process is exiting.
    virtual void exit() = 0;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_ENVIRONMENT_H
