/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_EVENTHANDLER_H
#define SSTMAC_SIMULATOR_EVENTHANDLER_H

#include <sstmac/simulator/timestamp.h>
#include <sstmac/util/nocopy.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

namespace sstmac {

  /**
   * Base type for event handlers that get called when the time
   * for an event arrives.
   */
  class eventhandler : virtual public nocopy {
  public:
    virtual ~eventhandler() throw() {}

    /// Your time has come.
    virtual void handle(const timestamp &completion_time) = 0;

    /// an event handler that gets ignored.
    static boost::shared_ptr<eventhandler> ignore;
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_EVENTHANDLER_H
