/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_EVENTMAP_H
#define SSTMAC_SIMULATOR_EVENTMAP_H

#include <sstmac/simulator/eventmanager.h>
#include <map>

namespace sstmac {
  
  /**
   * An event manager that relies on a multimap to manage events.
   */
  class eventmap : public eventmanager {
    /// The events sorted by timestamp.
    typedef std::multimap< timestamp, boost::shared_ptr<eventhandler> > queue_t;
    queue_t queue_;

    /// Sentinel to track whether the event handler is running or not.
    bool running_;

    /// Hello.
    eventmap(const timestamp &tv);

  public:
    /// Hello.
    static boost::shared_ptr<eventmap> construct(const timestamp &tv=timestamp(0));

    /// Goodbye.
    virtual ~eventmap() throw();

    /// Clear all events and set time back to a zero of your choice.
    /// \throw illformederror if the event manager is running.
    virtual void clear(const timestamp &zero_time = timestamp(0));

    /// Run the eventmanager.
    /// The eventmanager shall return control when no more messages remain.
    virtual void run();

    /// Set off the given eventhandler at the given time.
    virtual void schedule(const timestamp &start_time,
                          const boost::shared_ptr<eventhandler> &handler);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_EVENTMAP_H
