/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


#include <boost/shared_ptr.hpp>
#include <sstmac/simulator/fixedindexing.h>
#include <sstmac/util/errors.h>
#include <set>

namespace sstmac {
  
  using boost::shared_ptr;
  
  fixedindexing::fixedindexing(const std::vector<nodeid> &nodes):
  nodes_(nodes)
  {
  }

  shared_ptr<fixedindexing> fixedindexing::construct(const std::vector<nodeid>& nodes)
  {
    return shared_ptr<fixedindexing>(new fixedindexing(nodes));
  }
  
  void fixedindexing::allocate(const std::vector<nodeid> &nodes,
                               int ppn,
                               std::vector<nodeid> &result,
                               int nproc)
  {
    static std::string here("fixedindexing::allocate: ");
    if(nodes.empty()) {
      throw valueerror(here + "Empty node vector.");
    }
    if(ppn <= 0) {
      throw valueerror(here + "ppn must be > 0.");
    }
    if (nproc != nodes_.size()) {
      throw valueerror(here + "number of processes requested does not match"
                       " number of nodes available");
    }
    std::set<nodeid> nodes1, nodes2;
    nodes1.insert(nodes.begin(), nodes.end());
    nodes2.insert(nodes_.begin(), nodes_.end());
    if (nodes1 != nodes2) {
      throw valueerror(here+"given nodes do not match fixed nodes");
    }
    result = nodes_;
  }
  
} // end of namespace sstmac

