/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2011 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_FIXEDINDEXING_H 
#define SSTMAC_SIMULATOR_FIXEDINDEXING_H

#include <boost/shared_ptr.hpp>
#include <sstmac/simulator/indexstrategy.h>

namespace sstmac {

  class fixedindexing: public indexstrategy {
    std::vector<nodeid> nodes_;
  protected:
    fixedindexing(const std::vector<nodeid> &nodes);
  public:
    /// Construct a new object of this type.
    /// @param nodes the nodes to be allocated
    static boost::shared_ptr<fixedindexing>
    construct(const std::vector<nodeid>& nodes);
    
    virtual ~fixedindexing() throw() {}

    /// Assign processes to nodes using the fixed assignment
    /// vector passed to the constructor.
    /// \param nodes is the list of unique nodes to be used for the allocation
    /// \param ppn is the nominal number of processes allocated on each node.
    /// \param result is the resulting vector of length nodes (size nproc)
    /// \param nproc the total number of processes to allocate
    /// \throw valueerror if nodes.empty()
    /// \throw valueerror if ppn <= 0
    void allocate(const std::vector<nodeid> &nodes,
                  int ppn,
                  std::vector<nodeid> &result,
                  int nproc);
  };

} // end of namespace sstmac

#endif
