/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/networkinterface.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  using boost::shared_ptr;
  
  networkinterface::~networkinterface()
  {
  }

  void
  networkinterface::register_server(ppid caller,
                                      const shared_ptr<server> &serv)
  {
    if (has_server(caller))
    {
      throw illformederror("networkinterface::register_server: "
                           "already have a server defined.");
    }
    servers_[caller] = serv;
  }
  
  void
  networkinterface::unregister_server(ppid caller)
  {
    if (!has_server(caller))
    {
      throw nullerror("networkinterface::set_server: "
                      "no server defined");
    }
    servers_.erase(caller);
  }

  bool
  networkinterface::has_server(ppid caller) const
  {
    return (servers_.find(caller) != servers_.end());
  }
  
  shared_ptr<server>
  networkinterface::get_server(ppid caller)
  {
    servermap_t::iterator it = servers_.find(caller);
    if (it == servers_.end())
    {
      throw nullerror("networkinterface::get_server: No server defined "
                           "for the given ppid.");
    }
    return it->second;
  }
  
  void
  networkinterface::incoming_message(const timestamp &current_time,
                                     ppid caller,
                                     nodeid sender,
                                     const shared_ptr<networkdata> &load)
  {
    if (debugger::active())
      debug() << "networkinterface::incoming_message("
      << current_time << ", "
      << caller << ", " << sender << ", " << load << ")\n";
    shared_ptr<server> srv = get_server(caller);
    srv->incoming_message(current_time, sender, load);
  }

} // end of namespace sstmac.
