/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/node.h>
#include <sstmac/simulator/networkinterface.h>

namespace sstmac
{

  void
  node::initialize(nodeid my_id, const boost::shared_ptr<manager> &parent)
  {
    nodeid_ = my_id;
    parent_ = parent;
  }
  
  void
  node::add_networkinterface(const boost::shared_ptr<networkinterface> &nic)
  {
    nics_.push_back(nic);
  }

  boost::shared_ptr<networkinterface>
  node::networkinterface_for_server(ppid ppidarg)
  {
    for (size_t i=0; i<nics_.size(); i++) {
      if (nics_[i]->has_server(ppidarg)) {
        return nics_[i];
      }
    }
    throw nullerror("node::networkinterface_or_server: no nic found");
  }

} // end of namespace sstmac

