/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/nodelist.h>
#include <sstmac/simulator/node.h>
#include <sstmac/util/errors.h>

namespace sstmac {
  
  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi there.
  //
  nodelist::nodelist() {
  }

  //
  // Welcome to the new tomorrow.
  //
  shared_ptr<nodelist> nodelist::construct() {
    return shared_ptr<nodelist>(new nodelist);
  }

  //
  // The number of nodes in the list.
  //
  int64_t nodelist::size() const {
    return nodes_.size();
  }

  //
  // Get the given node in the list.
  //
  shared_ptr<node> nodelist::get(const nodeid &id) const {
    int64_t index = id.id;
    if(index < 0 || index >= int(nodes_.size())) {
      throw valueerror("nodelist::get:  Index out of range.");
    }
    return nodes_[index];
  }

  //
  // Add the given node to the list.
  //
  nodeid nodelist::add(const boost::shared_ptr<node> &the_node) {
    nodeid new_id(this->size());
    nodes_.push_back(the_node);
    return new_id;
  }

} // end of namespace sstmac
