/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef POWER_CONFIG_H_
#define POWER_CONFIG_H_

#include <boost/shared_ptr.hpp>
#include <ostream>

using namespace std;

namespace sstmac
{

  /// Used to store configuration info for everything power-related
  class power_config
  {
  protected:

    /// static POWERS per component, in watts (can expand these based on configurations,
    /// for example, rtr_static_per_port, or something like that)
    double proc_static;
    double mem_static;
    double rtr_static;

    /// dynamic ENERGIES per event, in joules
    double proc_flop, proc_memop, proc_memop_local, proc_memop_rand, mem_access;
    double rtr_hop_per_byte;

    /// Protected constructor. Initializes to default configuration.
    power_config();

    /// Protected constructor using a file.
    power_config(const std::string &filename);

  public:
    /// Public default constructor.
    static boost::shared_ptr<power_config>
    construct();

    /// Public constructor using a file.
    static boost::shared_ptr<power_config>
    construct(const std::string &filename);

    /// Farewell.
    ~power_config() throw ()
    {
    }

    //accessors
    double
    get_proc_flop()
    {
      return proc_flop;
    }
    double
    get_proc_memop()
    {
      return proc_memop;
    }
    double
    get_proc_memop_local()
    {
      return proc_memop_local;
    }
    double
    get_proc_memop_rand()
    {
      return proc_memop_rand;
    }
    double
    get_proc_static()
    {
      return proc_static;
    }
    double
    get_mem_static()
    {
      return mem_static;
    }

    double
    get_mem_access()
    {
      return mem_access;
    }

    double
    get_rtr_hop_per_byte()
    {
      return rtr_hop_per_byte;
    }
    double
    get_rtr_static()
    {
      return rtr_static;
    }

    void
    printPowerConfig(ostream &out);

  };

}

#endif /* POWER_CONFIG_H_ */
