/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef POWERMANAGER_H_
#define POWERMANAGER_H_

#include <boost/shared_ptr.hpp>

#include <sstmac/simulator/powerstat.h>
#include <list>
#include <ostream>
#include <string>

#include <sstmac/simulator/statsmanager.h>

using namespace std;

namespace sstmac
{

  /// Used to collect all the power stats in the simulation to add up at the end
  class powermanager : public statsmanager
  {
  public:

    virtual ~powermanager();

    static boost::shared_ptr<powermanager>
    construct(std::string fname)
    {
      boost::shared_ptr<powermanager> ret(new powermanager(fname));
      return ret;
    }

    /// Calculates the average power across the duration of the simulation
    virtual double
    getAveragePower(const timestamp &simtime);
    virtual double
    getAveragePower(const timestamp &simtime, powerstat::PWR_TYPE type);

    /// Calculates the total number of joules spent during the simulation
    virtual double
    getTotalEnergy(const timestamp &simtime);

    // Let the power manager know about this stat, so that he can collect it
    void
    registerPowerStat(boost::shared_ptr<powerstat> &stat);

    // Add up just the static powers (in Watts)
    double
    getTotalStatic();

    // Add up just the dynamic energy (in Joules)
    double
    getTotalDynamic();

    // prints the power stats
    virtual void
    printSummary(const timestamp &t);

  private:

    powermanager(std::string fname);


    list<boost::shared_ptr<powerstat> > stats;

  };

}

#endif
