/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/powerstat.h>

namespace sstmac
{
  powerstat::powerstat(PWR_TYPE t) : stats(stats::STAT_POWER)
  {
    energy = 0;
    power = 0;
    type = t;


  }

  powerstat::powerstat(const powerstat& orig) : stats(stats::STAT_POWER)
  {
    energy = orig.energy;
    power = orig.power;
  }

  powerstat::~powerstat()
  {
  }

  void
  powerstat::trackStaticPower(double watts)
  {

    //std::cerr << "STATS -- track power" << std::endl;
    power += watts;
  }

  void
  powerstat::trackEnergy(double joules)
  {


    energy += joules;
  }

  double
  powerstat::getAveragePower(const timestamp &simtime)
  {


    return power + (energy / simtime.sec());

  }

  double
  powerstat::getTotalEnergy(const timestamp &simtime)
  {
    return (power * simtime.sec()) + energy;

  }

}
