/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef POWERSTAT_H
#define	POWERSTAT_H

#include <sstmac/simulator/timestamp.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
#include <sstmac/simulator/stats.h>

namespace sstmac
{

  class powerstat : public stats
  {
  public:
    enum PWR_TYPE
    {
      PROCESSOR, MEMORY, NETWORK

    };

    virtual
    ~powerstat();

    static boost::shared_ptr<powerstat>
    construct(PWR_TYPE t)
    {
      boost::shared_ptr<powerstat> retval(new powerstat(t));
      return retval;
    }

    /// Track the static power of a component, understood to
    /// be consumed for the duration of the simulation
    void
    trackStaticPower(double watts);

    // Track a dynamic energy
    void
    trackEnergy(double joules);

    /// Calculates the average power across the duration of the simulation
    virtual double
    getAveragePower(const timestamp &simtime);

    /// Calculates the total number of joules spent during the simulation
    virtual double
    getTotalEnergy(const timestamp &simtime);

    double
    getStatic()
    {
      return power;
    }
    double
    getDynamic()
    {
      return energy;
    }
    PWR_TYPE
    getType()
    {
      return type;
    }

  protected:

    powerstat(PWR_TYPE t);
    powerstat(const powerstat& orig);

    double power; //static power
    double energy; // dynamic energy

    PWR_TYPE type;

  };

}

#endif	/* POWERSTAT_H */

