/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_SIMPLENIC_H
#define SSTMAC_SIMULATOR_SIMPLENIC_H

#include <sstmac/simulator/node.h>
#include <sstmac/simulator/networkinterface.h>
#include <sstmac/simulator/interconnect.h>

namespace sstmac {

  /**
   * A private nested type to handle sending data.
   */
  class simplenic : public networkinterface {
    boost::weak_ptr<node> parent_;
    boost::shared_ptr<interconnect> interconn_;
  protected:
    /// Hi.
    simplenic(const boost::shared_ptr<node> &parent,
              const boost::shared_ptr<interconnect> &interconn);

  public:
    /// Hello.
    static boost::shared_ptr<simplenic>
    construct(const boost::shared_ptr<node> &parent,
              const boost::shared_ptr<interconnect> &interconn);

    /// Goodbye.
    virtual ~simplenic() throw();

    /// Schedule a send operation to happen at the given time.
    /// \param start_time is the time when the send is started.
    /// \param ppidarg is the parallel process id of the source and target.
    /// \param recver is the global index of the receiver node.
    /// \param payload is the data sent over the network
    /// \param completion is the eventhandler signaled when the send is complete
    virtual void send(const timestamp &start_time,
                      ppid ppidarg,
                      nodeid recver,
                      const boost::shared_ptr<networkdata> &payload,
                      const boost::shared_ptr<eventhandler> &completion);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_SIMPLENIC_H
