/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include "statscollector.h"

namespace sstmac
{

  statscollector::statscollector(std::string fname)
  {
    pwr = powermanager::construct(fname);
    proc = procstatmanager::construct(fname);
    net = netstatmanager::construct(fname);
  }

  statscollector::~statscollector()
  {
    // TODO Auto-generated destructor stub
  }

  void
  statscollector::printStats(const timestamp &now)
  {
    pwr->printSummary(now);
    proc->printSummary(now);
    net->printSummary(now);
  }

  void
  statscollector::addStats(list<boost::shared_ptr<stats> > st)
  {
    list<boost::shared_ptr<stats> >::iterator it;


    for (it = st.begin(); it != st.end(); it++)
      {

        switch ((*it)->getType())
          {
        case stats::STAT_POWER:
          {
           // std::cout << "STATS -- adding power stat" << endl;
            boost::shared_ptr<powerstat> pstat = boost::dynamic_pointer_cast<
                powerstat>(*it);
            pwr->registerPowerStat(pstat);
            break;
          }

        case stats::STAT_PROC:
          {
           // std::cout << "STATS -- adding proc stat" << endl;
            boost::shared_ptr<procstats> prstat = boost::dynamic_pointer_cast<
                procstats>(*it);
            proc->registerStat(prstat);
            break;
          }

        case stats::STAT_NET:
          {
           // std::cout << "STATS -- adding net stat" << endl;
            boost::shared_ptr<netstat> nstat = boost::dynamic_pointer_cast<
                netstat>(*it);
            net->registerStat(nstat);
            break;
          }

          }
      }



  }

}
