/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef STATSCOLLECTOR_H_
#define STATSCOLLECTOR_H_

#include <string>

#include <sstmac/simulator/timestamp.h>

#include <sstmac/simulator/powermanager.h>
#include <sstmac/processor/procstatmanager.h>
#include <sstmac/network/netstatmanager.h>

#include <boost/shared_ptr.hpp>

using namespace std;

namespace sstmac
{
  class statscollector
  {
  public:
    virtual
    ~statscollector();
    static boost::shared_ptr<statscollector>
    construct(std::string fname)
    {
      boost::shared_ptr<statscollector> ret(new statscollector(fname));
      return ret;
    }

    void
    addStats(list<boost::shared_ptr<stats> > st);

    void printStats(const timestamp &now);

  private:

    statscollector(std::string fname);

    boost::shared_ptr<powermanager> pwr;
    boost::shared_ptr<procstatmanager> proc;
    boost::shared_ptr<netstatmanager> net;

  };

}

#endif /* STATSCOLLECTOR_H_ */
