/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_TASKID_H
#define SSTMAC_SIMULATOR_TASKID_H

#include <iostream>
#include <sstmac/util/cast.h>

namespace sstmac {

  /**
   * A ligthweight wrapper for a task index.
   * Done to avoid annoying mistakes where index values are mixed up.
   * 
   * I can't wait for C++ to add opaque typedefs.
   */
  struct taskid {
    int id;
    explicit taskid(int64_t v = -1) : id(check_cast<int>(v)) {}

    inline taskid& operator++() {
      ++id;
      return *this;
    }
    inline taskid operator++(int) {
      taskid other(*this);
      ++id;
      return other;
    }
  };

  inline bool operator==(const taskid &a, const taskid &b) {
    return (a.id == b.id);
  }
  inline bool operator!=(const taskid &a, const taskid &b) {
    return (a.id != b.id);
  }
  inline bool operator<(const taskid &a, const taskid &b) {
    return (a.id < b.id);
  }
  inline bool operator<=(const taskid &a, const taskid &b) {
    return (a.id <= b.id);
  }
  inline bool operator>(const taskid &a, const taskid &b) {
    return (a.id > b.id);
  }
  inline bool operator>=(const taskid &a, const taskid &b) {
    return (a.id >= b.id);
  }
  inline std::ostream& operator<<(std::ostream &os, const taskid &n) {
    return (os << "taskid(" << n.id << ")");
  }

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_TASKID_H
