/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/simulator/tasklist.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hello.
  //
  tasklist::tasklist(const std::vector<nodeid> &loc) :
    loc_(loc)
  {}

  //
  // Hi.
  //
  shared_ptr<tasklist> tasklist::construct(const std::vector<nodeid> &loc) {
    return shared_ptr<tasklist>(new tasklist(loc));
  }

  //
  // The number of tasks.
  //
  int64_t tasklist::size() const {
    return loc_.size();
  }

  //
  // The location of the given task.
  //
  const nodeid& tasklist::node(const taskid &index) const {
    if(index.id < 0 || index.id >= this->size()) {
      throw valueerror("tasklist::node:  index out of range.");
    }
    return loc_.at(index.id);
  }

} // end of namespace sstmac
