/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_DUMMY_DUMMYSERVER_H
#define SSTMAC_SKELETON_DUMMY_DUMMYSERVER_H

#include <sstmac/process/server.h>
#include <sstmac/simulator/ppid.h>
#include <map>

namespace sstmac {

  // Important types.
  class tasklist;
  class eventhandler;
  class taskid;
  class node;

  /**
   * A very basic network server for a dummy application.
   */
  class dummyserver : public server {
    /// The node this server is residing on.
    nodeid nodeid_;
    /// The task list (what node each task resides on).
    boost::shared_ptr<tasklist> tasklist_;
    /// My node interface.
    boost::shared_ptr<node> node_;
    /// My network interface.
    boost::shared_ptr<networkinterface> nic_;
    /// My parallel process id.
    ppid ppid_;

    /// The unique index that will be associated with the next ping.
    int next_ping_;
    /// My list of outstanding pings waiting for a pong.
    typedef std::map< int, boost::shared_ptr<eventhandler> > pending_t;
    pending_t pending_pong_;

  protected:
    /// Hi.
    dummyserver(const nodeid &mynode,
                const boost::shared_ptr<tasklist>& tlist,
                const ppid &ppidarg,
                const boost::shared_ptr<node> &nodearg,
                const boost::shared_ptr<networkinterface> &nic);

  public:
    /// Hello.
    static boost::shared_ptr<dummyserver>
    construct(const nodeid & mynode,
              const boost::shared_ptr<tasklist>& tlist,
              const ppid &ppidarg,
              const boost::shared_ptr<node> &nodearg,
              const boost::shared_ptr<networkinterface> &nic);

    /// Goodbye.
    virtual ~dummyserver() throw();

    /// Send a dummy message to another task in our run environment
    /// (actually gets ponged by the server that owns that task).
    /// When the response comes, back, notify the given eventhandler.
    void ping(const taskid &me, const taskid &task,
              const boost::shared_ptr<eventhandler>& notify);

    /// Complete a receive.
    /// Forwarded by the node when an incoming message arrives.
    /// It is the responsibility of the server to figure out how to
    /// understand any envelope information in the networkdata.
    virtual void incoming_message(const timestamp &start_time,
                                  const nodeid &sender,
                                  const boost::shared_ptr<networkdata>& load);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_DUMMY_DUMMYSERVER_H
