/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_FORCE_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_FORCE_H

#include <sstmac/skeleton/minimd/minimd.h>
#include <sstmac/process/runinterpolator.h>
#include <sstmac/process/mpiapi.h>

namespace sstmac {

  using boost::shared_ptr;

  /**
   * Take over the responsibilities of Force in miniMD.
   */
  class minimd::force {
  private:
    boost::shared_ptr<runinterpolator> execution_;
    boost::shared_ptr<mpiapi> mpi_;

  public:
    /// As in miniMD, all key variables are public.
    double cutforce;

    void init(const boost::shared_ptr<runinterpolator>& exe,
              const boost::shared_ptr<mpiapi>& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    void compute(const shared_ptr<atom>&, const shared_ptr<neighbor>&) {
      mpi_->compute(execution_->get("Force::compute"));
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_FORCE_H
