/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_IN_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_IN_H

#include <sstmac/skeleton/minimd/minimd.h>

namespace sstmac {

  // Ignored types.
  class runinterpolator;
  class mpiapi;

  /**
   * Take over the responsibilities of struct In in miniMD.
   */
  class minimd::in {
  public:
    /// Problem size.
    int nx, ny, nz;
    /// Target temperature.
    double t_request;
    /// Desired density.
    double rho;

    void init(const boost::shared_ptr<runinterpolator>&, const boost::shared_ptr<mpiapi>&) {
      // all in:: operations are rolled into runtime overhead for other calls.
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_IN_H
