/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_NEIGHBOR_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_NEIGHBOR_H

#include <sstmac/skeleton/minimd/minimd.h>
#include <sstmac/process/runinterpolator.h>
#include <sstmac/process/mpiapi.h>

namespace sstmac {

  /**
   * Take over the neighbor list management from miniMD.
   */
  class minimd::neighbor {
  private:
    boost::shared_ptr<runinterpolator> execution_;
    boost::shared_ptr<mpiapi> mpi_;

  public:
    // As in miniMD, all key variables are public.
    int every; /// reneighbor every n steps.
    int nbinx, nbiny, nbinz;  // number of global bins.
    double cutneigh; // neighbor cutoff.
    
    void init(const boost::shared_ptr<runinterpolator>& exe,
              const boost::shared_ptr<mpiapi>& mpi)
    {
      execution_ = exe;
      mpi_ = mpi;
    }

    /// This method is not in the real miniMD.
    void set_bins(int nx, int ny, int nz) {
      nbinx = nx; nbiny = ny; nbinz = nz;
    }

    /// This is a somewhat extensive function in the original.
    void setup() {
      mpi_->compute(execution_->get("Neighbor::setup"));
    }

    /// Build a neighbor list.
    void build(const boost::shared_ptr<atom>&) {
      mpi_->compute(execution_->get("Neighbor::build"));
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_NEIGHBOR_H
