/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

%{
namespace sstmac {}
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
#include <boost/lexical_cast.hpp>
using namespace sstmac;
%}

typedef unsigned long size_t;

%include exception.i
%{
static char errstr[8192];
%}
%exception {
  try {
    $action
  } catch(boost::bad_lexical_cast &exp) {
    sprintf(errstr, "boost::bad_lexical_cast thrown: %s (converting %s to %s)\n",
            exp.what(), exp.source_type().name(), exp.target_type().name());
    SWIG_exception(SWIG_RuntimeError, errstr);
  } catch(std::exception& exp) {
    sprintf(errstr, "std::exception thrown: %s\n", exp.what());
    SWIG_exception(SWIG_RuntimeError, errstr);
  } catch(const char* exp) {
    sprintf(errstr, "Exception thrown: %s\n", exp);
    SWIG_exception(SWIG_RuntimeError, errstr);
  }
  catch(...) {
    SWIG_exception(SWIG_RuntimeError,"Unknown Exception");
  }
}

// We need to get some key types right
typedef unsigned int uint32_t;
typedef long long int int64_t;
typedef unsigned long long int uint64_t;

%include std_string.i

%include boost_shared_ptr.i

// Swig goes a little crazy and declares all operator<< as "left shift".
%ignore sstmac::operator <<;
%ignore sstmac::operator <<;

// Do not warn that operator++ is ignored.
#pragma SWIG nowarn=SWIGWARN_IGNORE_OPERATOR_PLUSPLUS

// Do not warn that operator[] is ignored.
#pragma SWIG nowarn=SWIGWARN_IGNORE_OPERATOR_INDEX

// Do not warn that members that are python keywords are given a
// leading underscore in the python bindings (for example print
// becomes _print and from becomes _from).
#pragma SWIG nowarn=SWIGWARN_PARSE_KEYWORD

// Do not warn that operator= is ignored.
#pragma SWIG nowarn=SWIGWARN_IGNORE_OPERATOR_EQ

// Do not warn that from is renamed to _from.
#pragma SWIG nowarn=SWIGWARN_PARSE_KEYWORD
