/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


#ifndef SSTMAC_UTIL_ALLOC_H
#define SSTMAC_UTIL_ALLOC_H

#include <stdlib.h>
#include <iostream>

namespace sstmac {

  /// Allocated aligned memory using posix_memalign if supported. Otherwise malloc is used.
  /// The alignment must be a power of two. The default alignment is 8 bytes.
  template <class T>
  T *
  allocate_aligned(size_t alignment=8) {
    T *memptr;
#if _POSIX_C_SOURCE >= 200112 || _XOPEN_SOURCE >= 600
    int rv = posix_memalign(reinterpret_cast<void**>(&memptr), alignment, sizeof(T));
    if(rv) {
      std::cerr << "posix_memalign:  Failed to allocate block of size "
                << sizeof(T) << "\n";
      abort();
    }
#else
    (void)alignment;
    memptr = (T*)malloc(sizeof(T));
    if(memptr == 0) {
      std::cerr << "malloc:  Failed to allocate block of size "
                << sizeof(T) << "\n";
      abort();
    }
#endif
    return memptr;
  }

}

#endif
