/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_UTIL_CAST_H
#define SSTMAC_UTIL_CAST_H

#include <sstmac/util/errors.h>
#include <limits>

namespace sstmac {
  
  template <class T, class S>
  inline T check_cast(const S &s) {
    if (s>std::numeric_limits<S>::max()) throw sstmac::rangeerror("overflow");
    if (s<std::numeric_limits<S>::min()) throw sstmac::rangeerror("underflow");
    return static_cast<T>(s);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_UTIL_CAST_H
