/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/sstmac.h>
#include <unistd.h> // getopt

namespace sstmac {

  using boost::shared_ptr;

  void dummyrun(int nodes, int ppn) {
    //debug().to_stream(std::cout); // send copious debugging output to stdout.
    shared_ptr<manager> mgr = manager::construct();
    mgr->set_interconnect(fullnetwork::construct(timestamp(1e-9), 125000000));
    mgr->set_eventmanager(eventmap::construct());
    mgr->add_nodes(simplenode::construct(int(1e10)), nodes);
    shared_ptr<dummylauncher> job = dummylauncher::construct();
    mgr->add_launcher(timestamp(1), job, nodes, ppn);
    timestamp t = mgr->run();
    std::cout << "end time is " << t << std::endl;
  }

}

void print_help(const char *name) {
  std::cerr << name << " [-h] [-n nodes] [-p ppn]"
            << std::endl;
}

int main(int argc, char **argv) {
  bool helpflag = false;
  int opt;
  int nodes = 1, ppn = 1; // one task talking to itself.
  while(!helpflag && (opt = getopt(argc, argv, "hn:p:")) != -1) {
    switch(opt) {
    case 'h':
      helpflag = true;
      break;
    case 'n':
      nodes = atoi(optarg);
      break;
    case 'p':
      ppn = atoi(optarg);
      break;
    }
  }
  if(! helpflag) {
    bool errflag = false;
    if(nodes <= 0) {
      std::cerr << "Invalid node count \"" << nodes << "\"\n";
      errflag = true;
    }
    if(ppn <= 0) {
      std::cerr << "Invalid ppn count \"" << ppn << "\"\n";
      errflag = true;
    }
    if(! errflag) {
      sstmac::dummyrun(nodes, ppn);
    }
  }
  else {
    print_help(argv[0]);
  }
}
