#include <sstmac/sstmac.h>
#include <iostream>

using namespace sstmac;
using boost::shared_ptr;

class testmpicomms : public mpiapp {
  inline void testrank(const mpiid &worldrank, const mpiid &expected,
                       const mpiid &subrank, const std::string &call)
  {
    if(subrank != expected) {
      std::cerr << call << " failed:  World rank is " << worldrank
                << " but sub-communicator has rank " << subrank << "\n";
      throw illformederror(call + ":  Error in newly created communicator.");
    }
  }

  testmpicomms() {}

public:
  static shared_ptr<testmpicomms> construct() {
    shared_ptr<testmpicomms> retval(new testmpicomms);
    retval->set_self(retval);
    return retval;
  }

  virtual ~testmpicomms() {
  }

  virtual shared_ptr<mpiapp> clone() {
    return construct();
  }

  virtual void run() {
    // Create a bunch of new communicators.
    mpi()->init();
    mpicomm world = mpi()->comm_world();
    mpicomm full, half, third, fourth;
    //if(world.rank().id == 0) std::cerr << "Dup world\n";    
    mpi()->comm_dup(world, full);
    testrank(world.rank(), world.rank(), full.rank(), "mpi::comm_dup");
    //if(world.rank().id == 0) std::cerr << "Split half\n";    
    mpi()->comm_split(world, world.rank().id%2, world.rank().id, half);
    testrank(world.rank(), mpiid(world.rank().id / 2), half.rank(),
             "mpi::comm_split (half)");
    //if(world.rank().id == 0) std::cerr << "Split third\n";    
    mpi()->comm_split(world, world.rank().id%3, world.rank().id, third);
    testrank(world.rank(), mpiid(world.rank().id / 3), third.rank(),
             "mpi::comm_split (third)");
    //if(world.rank().id == 0) std::cerr << "Split fourth\n";    
    mpi()->comm_split(half, half.rank().id%2, half.rank().id, fourth);
    testrank(world.rank(), mpiid(world.rank().id / 4), fourth.rank(),
             "mpi::comm_split (fourth)");
    mpi()->finalize();
  }
};

int main() {
  shared_ptr<testmpicomms> test = testmpicomms::construct();
  shared_ptr<mpilaunch> launcher = mpilaunch::construct(test);
  shared_ptr<manager> mgr = manager::construct();
  timestamp lat = timestamp::exact_nsec(10);
  bandwidth bw(10, bandwidth::GBps);
  shared_ptr<network> net = crossbar::construct(256);
  shared_ptr<circuit> circ = circuit::construct(lat, bw, net);
  mgr->set_interconnect(circ);
  shared_ptr<simplenode> node 
    = simplenode::construct(lat/10, bw.get_bandwidth(bandwidth::Bps)*10, 1);
  mgr->add_nodes(node, 256);
  mgr->add_launcher(timestamp(0), launcher, 256, 1);
  timestamp ts = mgr->run();
  std::cout << "Completion time: " << ts << "\n";
}
