#include <sstmac/sstmac.h>
#include <iostream>

using namespace sstmac;
using boost::shared_ptr;

class testmpimagic : public mpiapp {
  testmpimagic() {}

public:
  static shared_ptr<testmpimagic> construct() {
    shared_ptr<testmpimagic> retval(new testmpimagic);
    retval->set_self(retval);
    return retval;
  }

  virtual ~testmpimagic() {
  }

  virtual shared_ptr<mpiapp> clone() {
    return construct();
  }

  virtual void run() {
    // Create a bunch of new communicators.
    mpi()->init();
    mpicomm world = mpi()->comm_world();
    mpicomm full, half, third, fourth;
    if(world.rank().id == 0) std::cerr << "Dup world\n";    
    mpi()->comm_dup(world, full);
    if(world.rank().id == 0) std::cerr << "Split half\n";    
    mpi()->comm_split(world, world.rank().id%2, world.rank().id, half);
    if(world.rank().id == 0) std::cerr << "Split third\n";    
    mpi()->comm_split(world, world.rank().id%3, world.rank().id, third);
    if(world.rank().id == 0) std::cerr << "Split fourth\n";    
    mpi()->comm_split(world, world.rank().id%4, world.rank().id, fourth);
    // Do magic barriers.
    shared_ptr<mpimagic> magic = mpi()->magic();
    shared_ptr<scheduler> sched = this->get_scheduler();
    mpitag tag(120);
    if(world.rank().id >= 0) std::cerr << "World barrier on node " << world.rank() << "\n";
    magic->barrier(tag, world, sched);
    if(world.rank().id >= 0) std::cerr << "Full barrier on node " << world.rank() << "\n";
    magic->barrier(tag, full, sched);
    if((world.rank().id % 2) == 0) {
      if(world.rank().id >= 0) std::cerr << "Half barrier on node " << world.rank() << "\n";
      magic->barrier(tag, half, sched);
    }
    if((world.rank().id % 3) == 0) {
      if(world.rank().id >= 0) std::cerr << "Third barrier on node " << world.rank() << "\n";
      magic->barrier(tag, third, sched);
    }
    if(world.rank().id >= 0) std::cerr << "Fourth barrier on node " << world.rank() << "\n";
    magic->barrier(tag, fourth, sched);
    if((world.rank().id % 3) != 0) {
      if(world.rank().id >= 0) std::cerr << "Third barrier on node " << world.rank() << "\n";
      magic->barrier(tag, third, sched);
    }
    if((world.rank().id % 2) != 0) {
      if(world.rank().id >= 0) std::cerr << "Half barrier on node " << world.rank() << "\n";
      magic->barrier(tag, half, sched);
    }
    if(world.rank().id >= 0) std::cerr << "Full barrier on node " << world.rank() << "\n";
    magic->barrier(tag, full, sched);
    mpi()->finalize();
  }
};

int main() {
  shared_ptr<testmpimagic> test = testmpimagic::construct();
  shared_ptr<mpilaunch> launcher = mpilaunch::construct(test);
  shared_ptr<manager> mgr = manager::construct();
  timestamp lat = timestamp::exact_nsec(10);
  bandwidth bw(10, bandwidth::GBps);
  shared_ptr<network> net = crossbar::construct(256);
  shared_ptr<circuit> circ = circuit::construct(lat, bw, net);
  mgr->set_interconnect(circ);
  shared_ptr<simplenode> node 
    = simplenode::construct(lat/10, bw.get_bandwidth(bandwidth::Bps)*10, 1);
  mgr->add_nodes(node, 256);
  mgr->add_launcher(timestamp(0), launcher, 256, 1);
  timestamp ts = mgr->run();
  std::cout << "Completion time: " << ts << "\n";
}
