/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/sstmac.h>
#include <unistd.h> // getopt
#include <string>

using boost::shared_ptr;
using namespace sstmac;

void
punt()
{
  std::cerr << "Error: usage: testsimpleapp [--config filename]\n";
  abort();
}

int
main(int argc, char* argv[])
{

  //debug().to_stderr();

  size_t stack = 1 << 15;
  int nodes = 2;

  // get config object for the processor model
  boost::shared_ptr<instructionprocessor_config> proc_cfg;

  if (argc == 1)
    proc_cfg = instructionprocessor_config::construct("PAPI_FP_INS",
        timestamp::exact_nsec(10), "PAPI_LST_INS", timestamp::exact_nsec(250));
  else if (argc == 3)
    {
      if (!std::string(argv[1]).compare("--config"))
        proc_cfg = instructionprocessor_config::construct(argv[2]);
      else
        punt();
    }
  else
    punt();

  shared_ptr<manager> mgr = manager::construct();

  mgr->set_interconnect(fullnetwork::construct(timestamp(1e-9), 125000000));

  mgr->set_eventmanager(eventmap::construct());

  timestamp lat(0);
  boost::shared_ptr<simplenode> node = simplenode::construct(lat, int(1e10),
     proc_cfg, 2);

  std::cerr << "initialized" << endl;

  mgr->add_nodes(node, nodes);

  shared_ptr<applaunch> job(applaunch::construct());
  std::map<taskid, boost::shared_ptr<app> > apps;
  for (int i = 0; i < nodes; ++i)
    apps[taskid(i)] = simpleapp::construct();
  job->set_applications(apps);
  job->set_scheduler(scheduler::construct(stack, stack * nodes, false));
  mgr->add_launcher(timestamp(1), job, nodes, 1);
  timestamp t = mgr->run();
  std::cout << "end time is " << t << std::endl;

}

