/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/sstmac.h>
#include <unistd.h> // getopt
using boost::shared_ptr;
using namespace sstmac;

int
main()
{

  //debug().to_stderr();

  size_t stack = 1 << 15;
  int nodes = 2;

  // construct environment
  shared_ptr<manager> mgr = manager::construct();
  mgr->set_interconnect(fullnetwork::construct(timestamp(1e-9), 125000000));
  mgr->set_eventmanager(eventmap::construct());
  timestamp lat(0);
  shared_ptr<simplenode> node = simplenode::construct(lat, int(1e10),
      instructionprocessor_config::construct());
  node->initialize(nodeid(), mgr);
  shared_ptr<nodeinterface> interface = node->get_nodeinterface(ppid(0),
      taskid(0));

  mgr->add_nodes(node, nodes);

  // construct launcher
  shared_ptr<mpiapp> application(simplempiapp::construct());
  shared_ptr<computeapi> api = computeapi::construct(application, interface);
  application->init_computeapi(api);
  shared_ptr<mpilaunch> job = mpilaunch::construct(application, true);
  job->set_scheduler(scheduler::construct(stack, stack * nodes, false));
  mgr->add_launcher(timestamp(1), job, nodes, 1);

  // run it
  mgr->run();

}

