/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009-2010 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/sstmac.h>
#include <unistd.h> // getopt
namespace sstmac
{

  using boost::shared_ptr;

  void
  simpleprocrun()
  {

    //debug().to_stderr();
    //debug().to_stream(std::cout);
    //debug().to_file("./output");
    boost::shared_ptr<manager> mgr = manager::construct();
    mgr->set_eventmanager(eventmap::construct());
    boost::shared_ptr<eventmanager> ev_manager = mgr->get_eventmanager();

    taskid tid;
    ppid pid;
    nodeid nid;

    // construct a node
    timestamp lat(0);
    boost::shared_ptr<simplenode> node = simplenode::construct(lat, int(1e10),
        instructionprocessor_config::construct());

    boost::shared_ptr<powermanager> pwr_mgr = powermanager::construct(
        "power.log");

    node->initialize(nid, mgr);

    std::cerr << "initialized" << endl;

    // create eventdata objects
    boost::shared_ptr<eventdata> instructions1 = eventdata::construct();
    instructions1->set_event("PAPI_FP_INS", 1000);
    instructions1->set_event("PAPI_LST_INS", 5);
    boost::shared_ptr<eventdata> instructions2 = eventdata::construct();
    instructions2->set_event("PAPI_FP_INS", 2000);
    instructions2->set_event("PAPI_LST_INS", 100);

    // create computekernel's
    shared_ptr<key> key1 = key::construct();
    boost::shared_ptr<computekernel> kernel1 = computekernel::construct(key1,
        instructions1);
    shared_ptr<key> key2 = key::construct();
    boost::shared_ptr<computekernel> kernel2 = computekernel::construct(key2,
        instructions2);

    // schedule computekernel
    shared_ptr<eventhandler> handler1(
        kernelhandler::construct(kernel1, node->get_nodeinterface(pid, tid)));
    shared_ptr<eventhandler> handler2(
        kernelhandler::construct(kernel2, node->get_nodeinterface(pid, tid)));
    kernel1->add_listener(handler2);
    ev_manager->schedule(ev_manager->now(), handler1);

    ev_manager->run();

    std::cout << "time is now " << ev_manager->now() << std::endl;
  }

}

int
main(int argc, char **argv)
{
  sstmac::simpleprocrun();
}
