/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_PAYLOAD_H
#define SSTMAC_PROCESS_PAYLOAD_H

#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * Pure abstract base for payload that can be associated with a network send.
   */
  class payload {
  public:
    /// Goodbye.
    virtual ~payload() throw() {}

    /// Create a copy of this item.
    /// This is required because the MPI API makes a copy of the provided
    /// payload (otherwise we can't ensure that the receiver actually gets
    /// the data that was sent).
    virtual boost::shared_ptr<payload> clone() const = 0;

    /// Convenience function to get a null payload pointer.
    static boost::shared_ptr<payload> null() {
      return boost::shared_ptr<payload>();
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_PAYLOAD_H
